<#--*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2009 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and may be covered by U.S. and Foreign Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************-->
<?xml version="1.0" encoding="utf-8"?>
<Form xmlns="http://www.adobe.com/2006/mxml" width="400" height="300"
    creationComplete="_creationCompleteHandler()">

    <#-- Resource Bundles -->
    <Metadata>
    <#list util.getAllResourceBundles(entity, termFormatter) as bundle>
        [ResourceBundle("${bundle}")]
    </#list>
    </Metadata>
    <#-- Set some display properties. -->
    <#list entity.getProperties() as property>
    <#-- display property -->
    <#if property.isData() || (property.isDerived() && !property.isConstraint())>
    <#-- use style caption aspect for label if present, default to property name if not -->
    <#if property.getStyle()?? && property.getStyle().getMessageAspect("caption")??>
        <#assign captionVal>{valueObject == null? '${util.toTitleCase(property.getName())}' : valueObject._model.${property.getName()}Style.caption.message}</#assign>
    <#else>
        <#assign captionVal>${util.toTitleCase(property.getName())}</#assign>
    </#if>
    <#-- Use style description aspect for toolTip if present.-->
    <#if property.getStyle()?? && property.getStyle().getMessageAspect("description")??>
        <#assign toolTipVal>toolTip="{valueObject._model.${property.getName()}Style.description.message}"</#assign>
    <#else>
        <#assign toolTipVal> </#assign>
    </#if>
    <#-- Toggle red if validations are present. -->
    <#if property.getStyle()?? && (property.getStyle().getValidations().size() > 0)>
        <#assign colorToggle>color="{valueObject._model.${property.getName()}IsValid ? 0x000000 : 0xFF0000}"</#assign>
    <#else>
        <#assign colorToggle> </#assign>
    </#if>
    <#-- Set binding and enabled properties. -->
    <#assign valueBinding>"{valueObject.${property.getName()}}"</#assign>
    <#assign valueBindingDest>destination="valueObject.${property.getName()}"</#assign>
    <#if property.isData() && !(property.isGenerated())>
        <#assign editableProp>enabled="{valueObject != null}"</#assign>
    <#else>
        <#assign editableProp>editable="false"</#assign>
    </#if>
    <#-- String FormItem -->
    <#if property.getType().getExpression() == "string" || property.getType().getExpression() == "char" || (property.getType().isNumericType() && !property.isData())>
    <FormItem label="${captionVal}" includeInLayout="{valueObject == null || valueObject._model.is${property.getName()?cap_first}Available}"
        visible="{valueObject == null || valueObject._model.is${property.getName()?cap_first}Available}">
        <TextInput id="_${property.getName()}Input" text=${valueBinding} ${colorToggle} ${editableProp} ${toolTipVal}/>
    </FormItem>
    <#if property.isData() && !(property.isGenerated())>
    <Binding source="_${property.getName()}Input.text" ${valueBindingDest}/>
    </#if>

    <#-- Numeric FormItem -->
    <#elseif property.getType().isNumericType()>
    <#if property.isData() && !(property.isGenerated())>
    <FormItem label="${captionVal}" includeInLayout="{valueObject == null || valueObject._model.is${property.getName()?cap_first}Available}"
        visible="{valueObject == null || valueObject._model.is${property.getName()?cap_first}Available}">
        <#if property.isId()>
            <#assign numericStepperMin>1</#assign>
            <#assign editableProp>enabled="{valueObject != null &amp;&amp; _serviceWrapper.serviceControl.commitRequired}"</#assign>
            <#assign stepSizeValue>1</#assign>
        <#else>
            <#if property.getType().getExpression() == "double" || property.getType().getExpression() == "float">
                <#assign stepSizeValue>0.1</#assign>
            <#else>
                <#assign stepSizeValue>1</#assign>
            </#if>
            <#assign numericStepperMin>-1000000</#assign>
            <#assign editableProp>enabled="{valueObject != null}"</#assign>
        </#if>
        <NumericStepper id="_${property.getName()}Input" minimum="${numericStepperMin}" stepSize="${stepSizeValue}" maximum="10000000"  ${colorToggle} value=${valueBinding} ${editableProp} ${toolTipVal}/>
    </FormItem>
    <Binding source="_${property.getName()}Input.value" ${valueBindingDest}/>
    </#if>

    <#-- Date FormItem -->
    <#elseif property.getType().getExpression() == "date">
    <FormItem label="${captionVal}" includeInLayout="{valueObject == null || valueObject._model.is${property.getName()?cap_first}Available}"
        visible="{valueObject == null || valueObject._model.is${property.getName()?cap_first}Available}">
    <#if property.isData() && !(property.isGenerated())>
        <DateField id="_${property.getName()}Input" yearNavigationEnabled="true"
            selectedDate=${valueBinding} ${editableProp} ${toolTipVal} ${colorToggle}/>
    <#else>
        <TextInput id="_${property.getName()}Input" text="{valueObject.${property.getName()}.toLocaleDateString()}" ${colorToggle} ${editableProp} ${toolTipVal}/>
    </#if>
    </FormItem>
    <#if property.isData() && !(property.isGenerated())>
    <Binding source="_${property.getName()}Input.selectedDate" ${valueBindingDest}/>
    </#if>

    <#-- Boolean FormItem -->
    <#elseif property.getType().getExpression() == "boolean">
    <FormItem label="${captionVal}" includeInLayout="{valueObject == null || valueObject._model.is${property.getName()?cap_first}Available}"
        visible="{valueObject == null || valueObject._model.is${property.getName()?cap_first}Available}">
        <CheckBox id="_${property.getName()}Input" selected=${valueBinding} ${editableProp} ${colorToggle} ${toolTipVal}/>
    </FormItem>
    <Binding source="_${property.getName()}Input.selected" ${valueBindingDest}/>

    <#-- Entity FormItem -->
    <#elseif property.getType().isEntity()>
    <#if property.getType().getAnnotationItem("ActionScriptGeneration", "DisplayColumn")??>
    <#assign labelFieldValue>${property.getType().getAnnotationItem("ActionScriptGeneration", "DisplayColumn")}</#assign>
    <#else>
    <#assign labelFieldValue>name</#assign>
    </#if>
    <FormItem label="${captionVal}" includeInLayout="{valueObject == null || valueObject._model.is${property.getName()?cap_first}Available}"
        visible="{valueObject == null || valueObject._model.is${property.getName()?cap_first}Available}">
        <ComboBox id="_${property.getName()}Input"
            ${editableProp} prompt="Select ${captionVal}"
            selectedItem="{valueObject.${property.getName()}}"
            labelField="${labelFieldValue}"/>
    </FormItem>
    <Binding source="_${property.getName()}Input.selectedItem as ${property.getType().getName()}" destination="valueObject.${property.getName()}"/>
    </#if>
    </#if>
    </#list>

    <#-- List of all constraint and validation failures -->
    <Spacer height="10"/>
    <#-- Add, Delete, Reset, and Save buttons -->
    <HBox width="100%" horizontalAlign="center">
        <Button id="_btnAdd" label="Add" click="_btnHandler(event)"
            enabled="{!_serviceWrapper.serviceControl.commitRequired}"
            icon="@Embed(source='icons/LC_Form_Add.png')"
            disabledIcon="@Embed(source='icons/LC_Form_Add_Disabled.png')"/>
        <Button id="_btnDelete" label="Delete" click="_btnHandler(event)"
            enabled="{valueObject != null &amp;&amp; !_serviceWrapper.serviceControl.commitRequired}"
            icon="@Embed(source='icons/LC_Form_Delete.png')"
            disabledIcon="@Embed(source='icons/LC_Form_Delete_Disabled.png')"/>
        <Button id="_btnReset" label="Reset" click="_btnHandler(event)"
            enabled="{_serviceWrapper.serviceControl.commitRequired}"
            icon="@Embed(source='icons/LC_Form_Reset.gif')"
            disabledIcon="@Embed(source='icons/LC_Form_Reset_Disabled.gif')"/>
        <Button id="_btnSave" label="Save" click="_btnHandler(event)"
            rollOver="_calculateErrorString()"
            enabled="{_serviceWrapper.serviceControl.commitRequired &amp;&amp; valueObject._model.isValid}"
            borderColor="{valueObject == null || valueObject._model.isValid? 0xaab3b3 : 0xff0000}"
            toolTip="{!valueObject._model.isValid? 'Save is not allowed due to:\n'  + _errorString : ''}"
            icon="@Embed(source='icons/LC_Form_Save.png')"
            disabledIcon="@Embed(source='icons/LC_Form_Save_Disabled.png')"/>
    </HBox>

    <Script>
    <![CDATA[
        import mx.events.CloseEvent;
        import mx.rpc.AsyncResponder;
        import mx.rpc.AsyncToken;
        import mx.rpc.events.ResultEvent;
        import mx.rpc.events.FaultEvent;
        import mx.controls.Alert;

        // Imports for the entity and its service.
        <#if packageMap.get(entity.getName())??>
        import ${packageMap.get(entity.getName())}.${entity.getName()};
        import ${packageMap.get(entity.getName())}.${entity.getName()?cap_first}Service;
        <#else>
        import ${model.name}.${entity.getName()};
        import ${model.name}.${entity.getName()?cap_first}Service;
        </#if>

        // Imports for associated entities and their services.
       <#list util.getAllToOneAssociatedEntities(entity).entrySet() as entry>
       <#if packageMap.get(entry.getKey().getName())??>
       import ${packageMap.get(entry.getKey().getName())}.${entry.getKey().getName()};
       import ${packageMap.get(entry.getKey().getName())}.${entry.getKey().getName()?cap_first}Service;
       <#else>
       import ${model.name}.${entry.getKey().getName()};
       import ${model.name}.${entry.getKey().getName()?cap_first}Service;
       </#if>
       </#list>

        [Bindable]
        private var _serviceWrapper:${entity.getName()?cap_first}Service;
        private var _valueObject:${entity.getName()};
        private var _prevValueObject:${entity.getName()};

        [Bindable]
        private var _errorString:String;

        [Bindable]
        public function get valueObject():${entity.getName()}
        {
            return _valueObject;
        }

        public function set valueObject(value:${entity.getName()}):void
        {
            _valueObject = value;
            _initValidations();
        }

        // Button handler.
        private function _btnHandler(event:Event):void
        {
            if (event.target == _btnAdd)
            {
                // Hold onto the current value object, in case add is reverted later.
                _prevValueObject = valueObject;

                valueObject = new ${entity.getName()}();

                <#list entity.getProperties() as property>
                <#if property.getType().getExpression() == "date" && property.isData() && !(property.isGenerated())>
                valueObject.${property.getName()} = new Date();
                </#if>
                </#list>

                <#list util.getAllToOneAssociatedEntities(entity).entrySet() as entry>
                <#list entry.getValue() as property>
                _${property.getName()}Input.selectedItem = null;
                </#list>
                </#list>
                _serviceWrapper.create${entity.getName()}(valueObject);
            }
            else if (event.target == _btnDelete)
            {
                Alert.show("Are you sure you want to delete the item?", "Delete Item", 3, this,
                    function alertClickHandler(event:CloseEvent):void
                    {
                        if (event.detail==Alert.YES)
                        {
                            var deleteToken:AsyncToken = _serviceWrapper.delete${entity.getName()}(valueObject);
                            deleteToken.addResponder(new AsyncResponder(
                                function (event:ResultEvent, token:Object=null):void
                                {
                                    valueObject = null;
                                },
                                function (event:FaultEvent, token:Object=null):void
                                {
                                    Alert.show("Delete failed: " + event.fault.faultString, "Error");
                                    _serviceWrapper.serviceControl.revertChanges(valueObject);
                                }));
                            _serviceWrapper.serviceControl.commit();
                        }
                    }
                );
            }
            else if (event.target == _btnSave)
            {
                <#list entity.getProperties() as property>
                <#if property.isData() && property.isId() && !(property.isGenerated())>
                // Set the id, if it is not auto generated.
                valueObject.${property.getName()} = _${property.getName()}Input.value;
                </#if>
                </#list>
                _serviceWrapper.serviceControl.commit(); // Commit the change.
                if (_prevValueObject != null)
                    _prevValueObject = null;
            }
            else if (event.target == _btnReset)
            {
                _serviceWrapper.serviceControl.revertChanges(valueObject);
                if (_prevValueObject != null)
                {
                    valueObject = _prevValueObject;
                    _prevValueObject = null;
                }
                else
                {
                    _initValidations();
                }
            }
        }

        // Creation complete handler.
        private function _creationCompleteHandler():void
        {
            // Initialize service wrapper and make sure autoCommit is set to false.
            _serviceWrapper = new ${entity.getName()}Service();
            _serviceWrapper.serviceControl.autoCommit = false;

            // Initialize associated services.
            <#list util.getAllToOneAssociatedEntities(entity).entrySet() as entry>
            <#list entry.getValue() as property>
            var ${property.getName()}Token:AsyncToken = new ${entry.getKey().getName()?cap_first}Service().getAll();
            ${property.getName()}Token.addResponder(new AsyncResponder(
                function (event:ResultEvent, token:Object=null):void
                {
                    _${property.getName()}Input.dataProvider= event.result;
                },
                function (event:FaultEvent, token:Object=null):void
                {
                    Alert.show("Associated service call failed: " + event.fault.faultString, "Error");
                }));
            </#list>
            </#list>
        }

        // Calculates the String for validation and constraint errors.
        private function _calculateErrorString():void
        {
            if (valueObject != null)
            {
                var errors:String = "";

                for (var i:int=0; i < valueObject._model.validationFailureMessages.length; i++)
                    errors += "-" + valueObject._model.validationFailureMessages[i] + ".\n";

                _errorString = errors;
            }
            else
            {
                _errorString = "";
            }
        }

        // Initializes style validators (if any) to the appropriate fields.
        private function _initValidations():void
        {
            if (valueObject == null)
                return;

            <#list entity.getProperties() as property>
            <#if property.getStyle()?? && (property.getStyle().getValidations().size() > 0)>
            valueObject._model.${property.getName()}Style.validator.source = _${property.getName()}Input;
            <#if property.getType().isNumericType()>
            valueObject._model.${property.getName()}Style.validator.property = "value";
            <#elseif property.getType().getExpression() == "string">
            valueObject._model.${property.getName()}Style.validator.property = "text";
            <#elseif property.getType().getExpression() == "date">
            valueObject._model.${property.getName()}Style.validator.property = "selectedDate";
            <#elseif property.getType().getExpression() == "boolean">
            valueObject._model.${property.getName()}Style.validator.property = "selected";
            </#if>
            </#if>
            </#list>
        }
    ]]>
    </Script>
</Form>