<#--*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2008 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and may be covered by U.S. and Foreign Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************-->
<#-- Constant for isManaged=true to enable freemarker scalar to freemarker scalar comparison -->
<#assign isYes>
yes
</#assign>
<#-- [Managed] vs [Bindable] depends on whether this class is data managed or not 
which in turn is determined by either a ServerType==LCDS annotation or a DataManagement annotation -->
<#assign isManaged>
<#if (entity.getAnnotationItem("ServerProperties", "ServerType")?? && entity.getAnnotationItem("ServerProperties", "ServerType") == "LCDS") || entity.getAnnotationGroup("DataManagement")??>
yes
<#else>
no
</#if>
</#assign>
<#-- Body of generated value object -->
<#assign classbody>
<#if isManaged == isYes>
[Managed]
</#if>
[ExcludeClass]
public class _Super_${entity.name} extends EventDispatcher implements IValueObject
{
    model_internal static function initRemoteClassAliasSingle(cz:Class) : void 
    {
        <#if util.getRemoteClassTag(model, entity)??>
        try 
        {
            if (flash.net.getClassByAlias("${util.getRemoteClassTag(model, entity)}") == null)
            {
                flash.net.registerClassAlias("${util.getRemoteClassTag(model, entity)}", cz);
            } 
        }
        catch (e:Error) 
        {
            flash.net.registerClassAlias("${util.getRemoteClassTag(model, entity)}", cz); 
        }
        </#if>
     }   
     
    model_internal static function initRemoteClassAliasAllRelated() : void 
    {     
        <#list entity.getAssociatedEntities(true, true) as assocEntity>
            <#if packageMap.get(assocEntity.getName())??>
         ${packageMap.get(assocEntity.getName())}.${assocEntity.getName()}.initRemoteClassAliasSingleChild();
            <#else>
         ${model.name}.${assocEntity.getName()}.initRemoteClassAliasSingleChild();
            </#if>   
        </#list>
    }

	model_internal var _dminternal_model : _${entity.name?cap_first}EntityMetadata;

	/**
	 * properties
	 */
<#assign atLeastOneStyle>false</#assign>	 
<#list entity.getProperties().iterator() as prop>
	<#if prop.isData()>
	<#-- honor opaque clarification annotation -->
		<#if prop.getAnnotationItem("OpaqueClarification", prop.getName())??>
	private var _internal_${prop.name} : ${typeHelper.format(prop.type, prop.getAnnotationItem("OpaqueClarification", prop.getName()))}<#if prop.hasDefaultValue()> = ${termFormatter.format(prop.getDefaultValueTerm)}</#if>;
		<#else>
	private var _internal_${prop.name} : ${typeHelper.format(prop.type)}<#if prop.hasDefaultValue()> = ${termFormatter.format(prop.getDefaultValueTerm())}</#if>;
		</#if>
	<#-- if we have a complex id property add a model_internal function so that we can insert null-guarded access later -->		
		<#if prop.isId() && prop.getType().isEntity()>
		
	model_internal function get internal_${prop.name}():${typeHelper.format(prop.type)}
	{
		return _internal_${prop.name};
	}
		
		</#if>		
	<#else>
	private var _internal_${prop.name} : ${typeHelper.format(prop.type)}
	</#if>
	<#if prop.isAggregation() || prop.isAssociation()>
	<#-- Pull in leaf type of collection types so serializer does not crash when attempting to instantiate them -->
		<#if prop.getType().isCollectionType()>
	model_internal var _internal_${prop.name}_leaf:${typeHelper.format(prop.getType().getLeafType())};
		</#if>
	</#if>
	<#if prop.getStyle()??>
		<#assign atLeastOneStyle>true</#assign>		
	</#if>
</#list>
<#-- clear opaque clarification from type formatter -->
<#if typeHelper.clearOpaqueClarification()></#if>

	<#-- declare single empty array variable so our reflection API does not have to return a new 
	instance every time it is called and an empty array needs to be returned-->
    private static var emptyArray:Array = new Array();
    <#-- Static variables for all services referenced in derived properties of this entity 
    static instances used when manging service does not return required service to us-->
<#list entity.getDependentOnServices() as service>
    model_internal static var static_${service.getName()}:${service.getName()} = null;
    <#-- make sure the service wrapper class is imported -->
    <#if typeHelper.addImport(service.getName(), false) == "_NONE_">
    </#if>
</#list>

    /**
     * derived property cache initialization
     */  
<#list entity.getDerivedProperties().iterator() as prop>
    model_internal var _cacheInitialized_${prop.name}:Boolean = false;
</#list>  
    model_internal var _cacheInitialized_isValid:Boolean = false;   
    
    <#-- Create an array of Change Watchers so that they do not get garbage collected -->
	model_internal var _changeWatcherArray:Array = new Array();   

	<#-- Constructor -->
	public function _Super_${entity.name}() 
	{	
		_model = new _${entity.name?cap_first}EntityMetadata(this);
	
		// Bind to own data properties for cache invalidation triggering  
	<#list entity.getDataProperties().iterator() as prop>
    	<#if (entity.getDependentProperties(prop, true).size() > 0 || entity.getGuardDependentProperties(prop, true).size() > 0
    		|| entity.getStyleDependentProperties(prop, true).size() > 0)>   
    		<#if prop.isData()>               
    	model_internal::_changeWatcherArray.push(ChangeWatcher.watch(this, "${prop.name}", model_internal::setterListener${prop.name?cap_first}));
    		<#else>
    	model_internal::_changeWatcherArray.push(ChangeWatcher.watch(this, "${prop.name}", model_internal::setterListener${prop.name?cap_first}));
    		</#if>        
        </#if> 
     </#list>    
       
     <#list entity.getConstraints().iterator() as constraint>             
	 	model_internal::_changeWatcherArray.push(ChangeWatcher.watch(this, "${constraint.name}", model_internal::setterListenerAnyConstraint));     
	 	model_internal::_changeWatcherArray.push(ChangeWatcher.watch(_model, "is${constraint.getName()?cap_first}Available", model_internal::setterListenerAnyConstraint));       
     </#list>           	
     <#-- if we have at least one style, want to watch the ResourceManager.localeChain property -->
     <#if "true" == atLeastOneStyle>
     	ResourceManager.getInstance().addEventListener("change", model_internal::setterListenerLocaleChain);
     </#if>
	}

    /**
     * data property getters
     */
<#list entity.getDataProperties().iterator() as prop>
	[Bindable(event="propertyChange")] 
	<#if prop.getAnnotationItem("OpaqueClarification", prop.getName())??>
    public function get ${prop.name}() : ${typeHelper.format(prop.type, prop.getAnnotationItem("OpaqueClarification", prop.getName()))}
    <#else>
    public function get ${prop.name}() : ${typeHelper.format(prop.type)}    
    </#if>
    {
            return _internal_${prop.name};
    }    
</#list>   
<#-- clear opaque clarification from type formatter -->
<#if typeHelper.clearOpaqueClarification()></#if> 

    /**
     * data property setters
     */      
<#list entity.getDataProperties().iterator() as prop>
	<#if prop.getAnnotationItem("OpaqueClarification", prop.getName())??>
    public function set ${prop.name}(value:${typeHelper.format(prop.type, prop.getAnnotationItem("OpaqueClarification", prop.getName()))}) : void 
    <#else>
    public function set ${prop.name}(value:${typeHelper.format(prop.type)}) : void 
    </#if>    
    {    	
        var recalcValid:Boolean = false;
    	<#-- trigger value object validity recalculation if this is a required property and its value
    	has changed either to null or from null.  Validity recalculation due to constraint value changes
    	is triggered by the ChangeWatchers defined above -->    	
    	<#if !(prop.isOptional()) && !(typeHelper.isASUnboxedType(prop.getType())) && !(prop.isId()) && !(prop.isVersion())>
    	if (value == null || _internal_${prop.name} == null)
    	{
    		recalcValid = true;
    	}	
        </#if>
    	
    	<#-- trigger valud object validity recalculation if there is a length limit on a string property -->
    	<#if (prop.getLength() > 0 && prop.getType().getExpression() == "string") >
    	recalcValid = true;
    	</#if>
    	
    	<#if prop.getAnnotationItem("OpaqueClarification", prop.getName())??>
    	var oldValue:${typeHelper.format(prop.type, prop.getAnnotationItem("OpaqueClarification", prop.getName()))} = _internal_${prop.name};               
    	<#else>
    	var oldValue:${typeHelper.format(prop.type)} = _internal_${prop.name};               
    	</#if>          
        if (oldValue !== value)
        {
           <#-- if this is an association or aggregation property, and derived properties reference a property
           of the associated/aggregated entity, we need to register change watchers for those related properties -->
           <#if (prop.isAggregation() || prop.isAssociation()) && !prop.getType().isCollectionType()>
                <#list entity.getDependentProperties(prop, false) as derivedProp>
                    <#list derivedProp.getPropertyReferences() as propRef>
                        <#if propRef.getBaseTerm().getClass().getName() == "fiber.core.impl.term.ref.PropertyReferenceImpl" && propRef.getBaseTerm().getPropertyName() == prop.getName()>
            model_internal::_changeWatcherArray.push(ChangeWatcher.watch(value, "${propRef.getPropertyName()}", model_internal::setterListener${prop.name?cap_first}));
                        </#if>
                    </#list>
                </#list>    
                <#list entity.getGuardDependentProperties(prop, false) as guardProp>
                    <#list guardProp.getGuardPropertyReferences() as propRef2>
                        <#if propRef2.getBaseTerm().getClass().getName() == "fiber.core.impl.term.ref.PropertyReferenceImpl" && propRef2.getBaseTerm().getPropertyName() == prop.getName()>
            model_internal::_changeWatcherArray.push(ChangeWatcher.watch(value, "${propRef2.getPropertyName()}", model_internal::setterListener${prop.name?cap_first}));
                        </#if>
                    </#list>
                </#list>   
                <#list entity.getStyleDependentProperties(prop, false) as styleProp>
                    <#list styleProp.getStyleExpressionPropertyReferences() as propRef3>
                        <#if propRef3.getBaseTerm().getClass().getName() == "fiber.core.impl.term.ref.PropertyReferenceImpl" && propRef3.getBaseTerm().getPropertyName() == prop.getName()>
            model_internal::_changeWatcherArray.push(ChangeWatcher.watch(value, "${propRef3.getPropertyName()}", model_internal::setterListener${prop.name?cap_first}));
                        </#if>
                    </#list>
                </#list> 
           </#if>
        	_internal_${prop.name} = value;
        	<#-- If the object is managed, compiler driven code generation due to the [Managed] metadata tag
        	     will take care of dispatching this event -->
        	<#if isManaged != isYes>
        	this.dispatchEvent(mx.events.PropertyChangeEvent.createUpdateEvent(this, "${prop.name}", oldValue, _internal_${prop.name}));
        	</#if>
        }    	     
        
        if (recalcValid && model_internal::_cacheInitialized_isValid)
        {
            model_internal::isValid_der = model_internal::calculateIsValid();
        }  
    }    
</#list>     
<#-- clear opaque clarification from type formatter -->
<#if typeHelper.clearOpaqueClarification()></#if>  

	<#-- Setter listeners are invoked when data properties are set via the ChangeWatchers above.  They in turn
	     trigger recalculation of all derived properties that depend on them.  They also add PropertyChangeEvent 
	     listeners that point to themselves for collection data properties -->  
    /**
     * data property setter listeners
     */   
<#list entity.getDataProperties().iterator() as prop>
   <#if (entity.getDependentProperties(prop, true).size() > 0) || (entity.getGuardDependentProperties(prop, true).size() > 0) 
   		|| (entity.getStyleDependentProperties(prop, true).size() > 0)>                      
   model_internal function setterListener${prop.name?cap_first}(value:Event):void
   {
        <#-- if this is a collection property, add event listener for collection changes -->
        <#if prop.getType().isCollectionType()>
        if (value is PropertyChangeEvent)
        {
        	if (PropertyChangeEvent(value).newValue)
            {
            	PropertyChangeEvent(value).newValue.addEventListener(CollectionEvent.COLLECTION_CHANGE, model_internal::setterListener${prop.name?cap_first});
            }
        } 
        </#if>        
        _model.invalidateDependentOn${prop.name?cap_first}();   
   }
   </#if> 
</#list>  

   <#-- Whenever any constraint changes value, the isValid property is recalculated -->
   model_internal function setterListenerAnyConstraint(value:Event):void
   {
        if (model_internal::_cacheInitialized_isValid)
        {
            model_internal::isValid_der = model_internal::calculateIsValid();
        }        
   }   

    /**
     * valid related derived properties
     */
    model_internal var _isValid : Boolean;
    model_internal var _invalidConstraints:Array = new Array();
    model_internal var _validationFailureMessages:Array = new Array();

    /**
     * derived property calculators
     */
<#-- derived property calculators differ depending on whether a call to a service is involved -->    
<#-- TODO: support multiple function references from the same derived property --> 
<#list entity.getDerivedProperties().iterator() as prop>
    <#if (prop.getServiceReferences(false, false).size() > 0)>
    <#-- derived property depends on a service invocation --> 
    model_internal function calculate${prop.name?cap_first}() : void
    {      
        <#list prop.getServiceInvocations().iterator() as invocation>
        <#assign serviceName = invocation.getTarget().getService().getName()/>                  
        <#if invocation.getTarget().getService().getAnnotationItem("ActionScriptGeneration", "ServiceType")??>          
        if (_managingService == null || _managingService.getService("${serviceName}") == null)
        <#else>
        if (_managingService == null || _managingService.getCustomService("${serviceName}") == null)
        </#if>
        {
			model_internal::initializeStatic${serviceName}();
        	model_internal::calculate${prop.name?cap_first}_viaWrapper(model_internal::static_${serviceName});
        }
        else
        {
        	<#if invocation.getTarget().getService().getAnnotationItem("ActionScriptGeneration", "ServiceType")??>
        	model_internal::calculate${prop.name?cap_first}_viaManagingService(_managingService.getService("${serviceName}"));
        	<#else>
        	model_internal::calculate${prop.name?cap_first}_viaWrapper(_managingService.getCustomService("${serviceName}") as ${serviceName});
        	</#if>
        }             
    }
    
    <#-- The following two methods allow us to just use the term formatter's output for a formatter invocation without
    having to add cases to it depending on how we got the service reference (managing service or statically).  AS
    does not support different local variable scopes in if-else blocks -->
    model_internal function calculate${prop.name?cap_first}_viaWrapper(${serviceName}:<#if packageMap.get(serviceName)??>${packageMap.get(serviceName)}<#else>${model.name}</#if>.${serviceName}):void
    {
    	var token:AsyncToken = ${termFormatter.format(invocation)};
        var responder:IResponder = new Responder(model_internal::calculate${prop.name?cap_first}R1, model_internal::calculate${prop.name?cap_first}F1);
        token.addResponder(responder);      	
    }
    
    model_internal function calculate${prop.name?cap_first}_viaManagingService(${serviceName}:AbstractService):void
    {
    	var token:AsyncToken = ${termFormatter.format(invocation)};
        var responder:IResponder = new Responder(model_internal::calculate${prop.name?cap_first}R1, model_internal::calculate${prop.name?cap_first}F1);
        token.addResponder(responder);        	
    }
    
    <#-- define result handler for service call --> 
    model_internal function calculate${prop.name?cap_first}R1(event:ResultEvent):void
    {
		var result:${typeHelper.format(prop.type)} = ${termFormatter.format(prop.valueTerm)?replace(termFormatter.format(invocation), "(event.result as " + typeHelper.format(prop.type) + ")")}; 
		model_internal::_cacheInitialized_${prop.name} = true; 
		model_internal::${prop.name}_der = result;  
    }      
 
    <#-- define fault handler for service call --> 
    model_internal function calculate${prop.name?cap_first}F1(event:FaultEvent):void
    {
		throw new Error("error during ${prop.name} calculation: " + event.fault.toString());    
    }
    	</#list>     
      
    <#else>
     <#-- derived property does not depend on service invocation --> 
    model_internal function calculate${prop.name?cap_first}() : void
    {     
        model_internal::_cacheInitialized_${prop.name} = true;
        model_internal::${prop.name}_der = ${termFormatter.format(prop.valueTerm)};
    }
    </#if>                   
</#list>  

    /**
     * isValid calculator
     */
    model_internal function calculateIsValid():Boolean
    {
        var violatedConsts:Array = new Array();    
        var validationFailureMessages:Array = new Array();    
<#list entity.getConstraints().iterator() as constraint>
    <#if constraint.hasGuard()>
        if (_model.is${constraint.getName()?cap_first}Available && !${constraint.name})
    <#else>
        if (!${constraint.name})
    </#if>
        {
            violatedConsts.push("${constraint.name}");
            <#if constraint.getStyle()?? && constraint.getStyle().getMessageAspect("error")??>
            validationFailureMessages.push(_model.${constraint.getName()}Style.error.message);           
            <#else>
            validationFailureMessages.push("${constraint.name} failed");
            </#if>
        }
</#list>    

<#-- required property testing -->
<#list entity.getRequiredProperties().iterator() as property>
    <#if !(property.isId()) && !(typeHelper.isASUnboxedType(property.getType())) && !(property.isVersion())>
		if (_model.is${property.getName()?cap_first}Available && _internal_${property.name} == null)
		{
			violatedConsts.push("${property.getName()}IsRequired");
			<#-- TODO: support localization, see LCDS-759 -->
			validationFailureMessages.push("${property.getName()} is required");
		}
    </#if>
</#list>  

		var styleValidity:Boolean = true;
<#list entity.getDataProperties() as prop>
    <#-- style validations -->
	<#if prop.getStyle()?? && (prop.getStyle().getValidations().size() > 0) >
		if (!_model.${prop.getName()}IsValid)
		{
			styleValidity = false;
			FiberUtils.arrayAdd(validationFailureMessages, _model.model_internal::_${prop.getName()}ValidationFailureMessages);
		}
	</#if>
	
	<#-- string property length testing -->
	<#if (prop.getLength() > 0 && prop.getType().getExpression() == "string") >
	   if (_model.is${prop.getName()?cap_first}Available && _internal_${prop.name} != null && _internal_${prop.name}.length > ${prop.getLength()?c})
	   {
            violatedConsts.push("${prop.getName()}IsTooLong");
            <#-- TODO: support localization, see LCDS-759 -->
            validationFailureMessages.push("${prop.getName()} is too long");	   
	   }
	</#if>
</#list>		
    
        model_internal::_cacheInitialized_isValid = true;
        model_internal::invalidConstraints_der = violatedConsts;
        model_internal::validationFailureMessages_der = validationFailureMessages;
        return violatedConsts.length == 0 && styleValidity;
    }  

<#-- derived properties have setters for two reasons.  One, we want to support server set derived properties
for properties that may be expensive or impossible to calculate on the client.  Two, it enables us to add
ChangeWatchers to them.  -->
    /**
     * derived property setters
     */
<#list entity.getDerivedProperties().iterator() as prop>
    model_internal function set ${prop.name}_der(value:${typeHelper.format(prop.type)}) : void 
    {
    	var oldValue:${typeHelper.format(prop.type)} = _internal_${prop.name};               
        if (oldValue !== value)
        {
        	_internal_${prop.name} = value;
        	this.dispatchEvent(mx.events.PropertyChangeEvent.createUpdateEvent(this, "${prop.name}", oldValue, _internal_${prop.name}));
        }
    }
</#list>

	<#-- special isValid property setter -->
    model_internal function set isValid_der(value:Boolean) : void
    {
       	var oldValue:Boolean = model_internal::_isValid;               
        if (oldValue !== value)
        {
        	model_internal::_isValid = value;
        	_model.model_internal::fireChangeEvent("isValid", oldValue, model_internal::_isValid);
        }        
    }

    /**
     * derived property getters
     */
<#list entity.getDerivedProperties().iterator() as prop>
	<#if isManaged == isYes>  
    [Transient] 
	</#if>
	[Bindable(event="propertyChange")] 
    public function get ${prop.name}() : ${typeHelper.format(prop.type)}
    {
        // check whether the derived property has been initialized
        if (model_internal::_cacheInitialized_${prop.name})
        {
            return _internal_${prop.name};
        }     
       else
        {
        <#-- do not put the auto recalculation on fist access in if DisableAutoReCalc is set for the property -->
        <#-- return default value for derived property's type instead -->
        <#if !((prop.getAnnotationItem("ActionScriptGeneration", "DisableAutoReCalc")?? && prop.getAnnotationItem("ActionScriptGeneration", "DisableAutoReCalc")=="true") || 
                 (entity.getAnnotationItem("ActionScriptGeneration", "DisableAutoReCalc")?? && entity.getAnnotationItem("ActionScriptGeneration", "DisableAutoReCalc")=="true"))>        
            // recalculate property
            model_internal::calculate${prop.name?cap_first}();
            return _internal_${prop.name};
        <#else>
            // auto-recalculation has been disabled for this derived property.  Returning default value for its type
            <#-- return default value for type if the type is unboxed in AS and we can't return null for it -->
            <#if typeHelper.isASUnboxedType(prop.type)>
            return ${termFormatter.format(prop.type.getDefaultValue())};
            <#else>
            return null;
            </#if>
        </#if>            
        }                
    }
</#list>    

    [Transient] 
	[Bindable(event="propertyChange")] 
    public function get _model() : _${entity.name?cap_first}EntityMetadata
    {
		return model_internal::_dminternal_model;              
    }	
    
    public function set _model(value : _${entity.name?cap_first}EntityMetadata) : void       
    {
    	var oldValue : _${entity.name?cap_first}EntityMetadata = model_internal::_dminternal_model;               
        if (oldValue !== value)
        {
        	model_internal::_dminternal_model = value;
        	this.dispatchEvent(mx.events.PropertyChangeEvent.createUpdateEvent(this, "_model", oldValue, model_internal::_dminternal_model));
        }     
    }      

<#-- Declare methods.  Their implementation must be provided via the customizable sub class -->
    /**
     * methods
     */  
<#list entity.getMethods().iterator() as method>
    protected function ${method.getName()} ${typeHelper.format(method.getSignature())}:${typeHelper.format(method.getSignature().getReturnType())}
    {
        throw new Error("${method.getName()} must be overridden");
    }
</#list>   

<#-- Lazy static service variable intializers -->
<#list entity.getDependentOnServices() as service>
    model_internal function initializeStatic${service.getName()}():void
    {
    	if (model_internal::static_${service.getName()} == null)
    	{
    		model_internal::static_${service.getName()} = new ${service.getName()}();
    	}    
    }
    
</#list>

<#-- Managing service variable -->
    /**
     *  services
     */                  
     private var _managingService:IFiberManagingService;
    
     public function set managingService(managingService:IFiberManagingService):void
     {
         _managingService = managingService;
     }                      
     
    model_internal function set invalidConstraints_der(value:Array) : void
    {  
     	var oldValue:Array = model_internal::_invalidConstraints;
     	// avoid firing the event when old and new value are different empty arrays
        if (oldValue !== value && (oldValue.length > 0 || value.length > 0))
        {
            model_internal::_invalidConstraints = value;   
			_model.model_internal::fireChangeEvent("invalidConstraints", oldValue, model_internal::_invalidConstraints);   
        }     	             
    }             
    
     model_internal function set validationFailureMessages_der(value:Array) : void
    {  
     	var oldValue:Array = model_internal::_validationFailureMessages;
     	// avoid firing the event when old and new value are different empty arrays
        if (oldValue !== value && (oldValue.length > 0 || value.length > 0))
        {
            model_internal::_validationFailureMessages = value;   
			_model.model_internal::fireChangeEvent("validationFailureMessages", oldValue, model_internal::_validationFailureMessages);   
        }     	             
    }        
     
     // Individual isAvailable functions     
<#-- isAvailable cache -->
<#list entity.getGuardedProperties().iterator() as prop>
    model_internal var _isAvailableCache_${prop.name}:Boolean = false;     
    model_internal var _isAvailable${prop.name?cap_first}:Boolean = false;
                    
</#list>  
<#list entity.getGuardedProperties().iterator() as prop>
    <#-- isAvailable setter.  Need setter so isAvailable can be bindable -->
    model_internal function set is${prop.getName()?cap_first}Available_der(value:Boolean):void
    {
        var oldValue:Boolean = model_internal::_isAvailable${prop.name?cap_first};               
        if (oldValue !== value)
        {
        	model_internal::_isAvailable${prop.name?cap_first} = value;
        	_model.model_internal::fireChangeEvent("is${prop.name?cap_first}Available", oldValue, model_internal::_isAvailable${prop.name?cap_first});  
        }                             
    }                     
     
     <#-- isAvailable calculator -->   
     model_internal function calculateIsAvailable${prop.name?cap_first}():void
     {
        model_internal::_isAvailableCache_${prop.name} = true;
        
        var oldValue:Boolean = model_internal::_isAvailable${prop.getName()?cap_first};
        var newValue:Boolean = ${termFormatter.format(prop.guardTerm)};
        
        model_internal::is${prop.getName()?cap_first}Available_der = newValue;
        
        <#if prop.isDerived()>
        // recalculate derived properties when they become available
        if (!oldValue && newValue)
        {
            model_internal::calculate${prop.name?cap_first}();
        }       
        </#if>   
        
        <#if prop.getStyle()??>
        // recalculate style attributes when a property's availability changes
        if (oldValue != newValue)
        {
           <#if (prop.getStyle().getValidations().size() > 0)>
            model_internal::_doValidationForStyleCacheOf${prop.getName()} = null;
           </#if>
            model_internal::_calculate${prop.name?cap_first}Style();
        }       
        </#if>           
     }     
                          
</#list>   
	// fields, getters, and setters for primitive representations of complex id properties
<#-- fields, getters, and setters for primitive representations of complex id properties -->
<#if (util.getPrimitiveIdPropsInfo(model, entity, false).entrySet().size() > 0)>
	<#list util.getPrimitiveIdPropsInfo(model, entity, false).entrySet() as idPropEntry>
	private var _internal_${idPropEntry.getKey()} : ${typeHelper.format(idPropEntry.getValue().getIdProperty().getType())}

	[Bindable(event="propertyChange")] 				
	public function get ${idPropEntry.getKey()}() : ${typeHelper.format(idPropEntry.getValue().getIdProperty().getType())}
	{				
		return ${modelInternalTermFormatter.format(idPropEntry.getValue().getNullTestTerm())} ? _internal_${idPropEntry.getKey()} : ${termFormatter.format(idPropEntry.getValue().getExpressionTerm())};
	}
	
	public function set ${idPropEntry.getKey()}(value : ${typeHelper.format(idPropEntry.getValue().getIdProperty().getType())}) : void
	{
		_internal_${idPropEntry.getKey()} = value;
	}	

	</#list>    
</#if>     
<#-- style calculators -->
<#list entity.getProperties().iterator() as prop>                 
	<#if prop.getStyle()??>	
	model_internal function _calculate${prop.getName()?cap_first}Style():void
	{
		if (_model.model_internal::_${prop.getName()}Style != null)
		{
		<#-- Message Aspects -->
		<#list prop.getStyle().getMessageAspects().entrySet() as entry>
			<#-- Bundle -->
			<#if entry.getValue().getBundleTerm()??>
			_model.model_internal::_${prop.getName()}Style.${entry.getKey()}.bundle = ${termFormatter.format(entry.getValue().getBundleTerm())};	
			<#else>
			_model.model_internal::_${prop.getName()}Style.${entry.getKey()}.bundle = "";				
			</#if>
			<#-- Text -->			
			<#if entry.getValue().getTextTerm()??>
			_model.model_internal::_${prop.getName()}Style.${entry.getKey()}.text = ${termFormatter.format(entry.getValue().getTextTerm())};	
			<#else>
			_model.model_internal::_${prop.getName()}Style.${entry.getKey()}.text = "";				
			</#if>		
			<#-- Key -->			
			<#if entry.getValue().getKeyTerm()??>
			_model.model_internal::_${prop.getName()}Style.${entry.getKey()}.key = ${termFormatter.format(entry.getValue().getKeyTerm())};	
			<#else>
			_model.model_internal::_${prop.getName()}Style.${entry.getKey()}.key = "";				
			</#if>	
			<#-- Tokens -->			
			<#if entry.getValue().getTokensTerm()??>
			_model.model_internal::_${prop.getName()}Style.${entry.getKey()}.tokens = ${termFormatter.format(entry.getValue().getTokensTerm())};	
			<#else>
			_model.model_internal::_${prop.getName()}Style.${entry.getKey()}.tokens = null;				
			</#if>			
			_model.model_internal::_${prop.getName()}Style.${entry.getKey()}.model_internal::calculateMessage();			
			<#-- if this property is a constraint and its error message has changed, we need to recalculate validity to get the right error message out there -->
			<#if prop.isDerived() && prop.isConstraint() && entry.getKey() == "error">
			model_internal::isValid_der = model_internal::calculateIsValid();
			</#if>		
		</#list>	
		<#-- Masks -->
		<#if prop.getStyle().getEditMask()??>
			_model.model_internal::_${prop.getName()}Style.editMask = ${termFormatter.format(prop.getStyle().getEditMask().getMaskTerm())};
		</#if>
		<#if prop.getStyle().getDisplayMask()??>
			_model.model_internal::_${prop.getName()}Style.displayMask = ${termFormatter.format(prop.getStyle().getDisplayMask().getMaskTerm())};
		</#if>
		<#-- Validations -->
		<#if (prop.getStyle().getValidations().size() > 0)>
			_model.model_internal::_${prop.getName()}Style.validator = _model.model_internal::_${prop.getName()}StyleValidator;
		</#if>		
		}
		<#-- Validation Calculation -->
		<#if (prop.getStyle().getValidations().size() > 0)>
			<#-- trigger asynchronous calculation of any validations that depend on service function calls,
			         only if it has been triggered before.  Otherwise, the validator will trigger it and we do not need
			         to duplicate the effort here -->
			 <#list prop.getStyle().getValidations() as validation>
	 			<#if validation.getServiceDependencies()?? && (validation.getServiceDependencies().size() > 0)>
	 	if (model_internal::_cacheInitialized_validation_${validation.uniqueIdentifier()})
	 	     model_internal::calculate_validation_${validation.uniqueIdentifier()}(${prop.getName()}) ;
	 			</#if>
	 		</#list>
		
		 // perform property level validation
		 var valRes:ValidationResultEvent = _model.model_internal::_${prop.getName()}StyleValidator.validate(${prop.getName()});
		 _model.model_internal::_${prop.getName()}IsValid_der = (valRes.results == null);
		 <#-- update property level validation failures array -->
		 if (valRes.results == null)
		      _model.model_internal::${prop.getName()}ValidationFailureMessages_der = emptyArray;
		 else
		 {
		      var _valFailures:Array = new Array();
		      for (var a:int = 0 ; a<valRes.results.length ; a++)
		      {
                    _valFailures.push(valRes.results[a].errorMessage);		              
		      }
		      _model.model_internal::${prop.getName()}ValidationFailureMessages_der = _valFailures;
		 }     

		</#if>		
	}
		<#if (prop.getStyle().getValidations().size() > 0)>
	<#-- if  a validation expression of a style depends on a service function call, we need plumbing to handle the asynchronous nature of the call -->
	<#list prop.getStyle().getValidations() as validation>
	
		<#if validation.getServiceDependencies()?? && (validation.getServiceDependencies().size() > 0)>
	<#-- variables for handling service functions refs in validations -->
	model_internal var _cacheInitialized_validation_${validation.uniqueIdentifier()}:Boolean = false;
	model_internal var _validation_${validation.uniqueIdentifier()}:Boolean = true;
	
	<#-- service invocation plumbing for service calls from validations -->
	model_internal function calculate_validation_${validation.uniqueIdentifier()}(value : ${typeHelper.format(prop.getType())}) : void
    {      
        	<#list validation.getServiceInvocations().iterator() as invocation>
        		<#assign serviceName = invocation.getTarget().getService().getName()/>                  
        		<#if invocation.getTarget().getService().getAnnotationItem("ActionScriptGeneration", "ServiceType")??>          
        if (_managingService == null || _managingService.getService("${serviceName}") == null)
        		<#else>
        if (_managingService == null || _managingService.getCustomService("${serviceName}") == null)
        		</#if>
        {
			model_internal::initializeStatic${serviceName}();
        	model_internal::calculate_validation_${validation.uniqueIdentifier()}_viaWrapper(model_internal::static_${serviceName}, value);
        }
        else
        {
        		<#if invocation.getTarget().getService().getAnnotationItem("ActionScriptGeneration", "ServiceType")??>
        	model_internal::calculate_validation_${validation.uniqueIdentifier()}_viaManagingService(_managingService.getService("${serviceName}"), value);
        		<#else>
        	model_internal::calculate_validation_${validation.uniqueIdentifier()}_viaWrapper(_managingService.getCustomService("${serviceName}") as ${serviceName}, value);
        		</#if>
        }             
    }
    
    <#-- The following two methods allow us to just use the term formatter's output for a formatter invocation without
    having to add cases to it depending on how we got the service reference (managing service or statically).  AS
    does not support different local variable scopes in if-else blocks -->
    model_internal function calculate_validation_${validation.uniqueIdentifier()}_viaWrapper(${serviceName}:<#if packageMap.get(serviceName)??>${packageMap.get(serviceName)}<#else>${model.name}</#if>.${serviceName}, value:${typeHelper.format(prop.getType())}):void
    {
    	var token:AsyncToken = ${validationTermFormatterFactory.getFormatter(prop.getName()).format(invocation)};
        var responder:IResponder = new Responder(model_internal::calculate_validation_${validation.uniqueIdentifier()}R1, model_internal::calculate_validation_${validation.uniqueIdentifier()}F1);
        token.addResponder(responder);      	
        token["value"] = value;
    }
    
    model_internal function calculate_validation_${validation.uniqueIdentifier()}_viaManagingService(${serviceName}:AbstractService, value:${typeHelper.format(prop.getType())}):void
    {
    	var token:AsyncToken = ${validationTermFormatterFactory.getFormatter(prop.getName()).format(invocation)};
        var responder:IResponder = new Responder(model_internal::calculate_validation_${validation.uniqueIdentifier()}R1, model_internal::calculate_validation_${validation.uniqueIdentifier()}F1);
        token.addResponder(responder);      
        token["value"] = value;  	
    }
    
    <#-- define result handler for service call --> 
    model_internal function calculate_validation_${validation.uniqueIdentifier()}R1(event:ResultEvent):void
    {
        <#-- clear the cache for the validation calculation -->
        model_internal::_doValidationForStyleCacheOf${prop.getName()} = null; 
        var value:${typeHelper.format(prop.getType())} = event.token["value"] as ${typeHelper.format(prop.getType())};
		var result:Boolean= ${validationTermFormatterFactory.getFormatter(prop.getName()).format(validation.getValidationTerm())?replace(validationTermFormatterFactory.getFormatter(prop.getName()).format(invocation), "(event.result as " + typeHelper.format(invocation.getType()) + ")")}; 
		model_internal::_cacheInitialized_validation_${validation.uniqueIdentifier()}= true; 
		model_internal::_validation_${validation.uniqueIdentifier()} = result;  
		
		<#-- an internal call to doValidation returns an array of ValidationResults -->
		var valRes : Array = model_internal::_doValidationForStyleOf${prop.getName()}(value);

         _model.model_internal::_${prop.getName()}IsValid_der = (valRes.length == 0);
         <#-- update property level validation failures array -->
         if (valRes.length == 0)
              _model.model_internal::${prop.getName()}ValidationFailureMessages_der = emptyArray;
         else
         {
              var _valFailures:Array = new Array();
              for (var a:int = 0 ; a<valRes.length ; a++)
              {
                    _valFailures.push(valRes[a].errorMessage);                    
              }
              _model.model_internal::${prop.getName()}ValidationFailureMessages_der = _valFailures;
         }   
    }      
 
    <#-- define fault handler for service call --> 
    model_internal function calculate_validation_${validation.uniqueIdentifier()}F1(event:FaultEvent):void
    {
		throw new Error("error during calculation of validation expression for property ${prop.getName()}: " + event.fault.toString());    
    }
    		</#list>
		</#if>
	</#list>
	
	model_internal var _doValidationForStyleCacheOf${prop.getName()} : Array = null;
	model_internal var _doValidationLastValForStyleOf${prop.getName()} : ${typeHelper.format(prop.getType())};
		
	<#-- style validation function -->
	model_internal function _doValidationForStyleOf${prop.getName()}(valueIn:Object):Array
	{
	    var value : ${typeHelper.format(prop.getType())} = valueIn as ${typeHelper.format(prop.getType())} ;

        if (model_internal::_doValidationForStyleCacheOf${prop.getName()} != null && model_internal::_doValidationLastValForStyleOf${prop.getName()} == value)
           return model_internal::_doValidationForStyleCacheOf${prop.getName()} ;

		_model.model_internal::_${prop.getName()}StyleValidCacheInitialized = true;
 		var validationFailures:Array = new Array();
 		var errorMessage:String;
 		var failure:Boolean;

 		 <#-- Handle each validation expression individually -->
 		 <#if typeHelper.addImport("mx.validators.ValidationResult", false) == "_NONE_">
         </#if>
        var valRes:ValidationResult;
 		<#list prop.getStyle().getValidations() as validation>
 			<#if validation.getAnnotationGroup("ActionScriptValidator")??>
 		var autoResult:ValidationResultEvent = _model.model_internal::_${prop.getName()}BuiltInStyleValidator.validate(value);
 		failure = (autoResult.results != null);
 			<#else>
 				<#-- if validaiton involves service function call need special handling -->
 				<#if validation.getServiceDependencies()?? && (validation.getServiceDependencies().size() > 0)>
 		if (!model_internal::_cacheInitialized_validation_${validation.uniqueIdentifier()})
 			model_internal::calculate_validation_${validation.uniqueIdentifier()}(value);
 			
 		failure = !(_validation_${validation.uniqueIdentifier()});
 				<#else>
 		failure = (!(${validationTermFormatterFactory.getFormatter(prop.getName()).format(validation.getValidationTerm())}));
 				</#if>
 			</#if>
 				
 		if (failure)
 		{
 			errorMessage = null;
 			<#-- trying going to resource manager first -->
 			<#if validation.getBundleTerm()?? && validation.getKeyTerm()??>
 				<#if validation.getTokensTerm()??>
 			errorMessage = ResourceManager.getInstance().getString(${termFormatter.format(validation.getBundleTerm())}, ${termFormatter.format(validation.getKeyTerm())}, ${termFormatter.format(validation.getTokensTerm())}.source);		
 				<#else>
  			errorMessage = ResourceManager.getInstance().getString(${termFormatter.format(validation.getBundleTerm())}, ${termFormatter.format(validation.getKeyTerm())});					
  				</#if>	
 			</#if>
 			
 			<#-- default to literal text term -->
 			if (errorMessage == null)
 			{
 			 	<#if validation.getAnnotationGroup("ActionScriptValidator")??>
 				errorMessage = autoResult.message;
 				if (errorMessage == null)
 				{
 				</#if>
 			
 				<#if validation.getTextTerm()??>
 				   <#if validation.getTokensTerm()??>
 				errorMessage = StringUtil.substitute(${termFormatter.format(validation.getTextTerm())}, ${termFormatter.format(validation.getTokensTerm())}.source);  				
 				    <#if typeHelper.addImport("mx.utils.StringUtil", false) == "_NONE_">
                    </#if>
 				   <#else>
                errorMessage = ${termFormatter.format(validation.getTextTerm())};                				   
 				   </#if>
 				<#else>
 				<#-- All else fail, English literal -->
 				errorMessage = "Validation for property '${prop.getName()}' failed";
 				</#if>
 			 	<#if validation.getAnnotationGroup("ActionScriptValidator")??>
				}
 				</#if> 				
 			}
 			
 			valRes = new ValidationResult(true, "", "", errorMessage);
 			validationFailures.push(valRes);
 		}
 		</#list> 	
 		
 	    model_internal::_doValidationForStyleCacheOf${prop.getName()} = validationFailures;
 	    model_internal::_doValidationLastValForStyleOf${prop.getName()} = value;
 		
 		return validationFailures;
	}
	
		</#if>	
	</#if>	             
</#list>    

<#-- ResourceManager localeChainListener -->
<#if "true" == atLeastOneStyle>
	model_internal function setterListenerLocaleChain(event:Event):void
	{	
	<#list entity.getProperties().iterator() as prop>                 
		<#if prop.getStyle()??>		
		<#-- first invalidate validation cache so that validation error messages are recalculated -->
		      <#if (prop.getStyle().getValidations().size() > 0)>
        model_internal::_doValidationForStyleCacheOf${prop.getName()} = null;
               </#if>
        
		if (_model.model_internal::_${prop.getName()}Style != null)
			model_internal::_calculate${prop.getName()?cap_first}Style();
		</#if>
	</#list>
	}
</#if>
<#if entity.getAnnotationItem("ActionScriptGeneration", "DisplayColumn")??>
	// Display column set as an annotation, so the toString() function is overridden to return that column
	override public function toString():String
	{
		return "" + this.${entity.getAnnotationItem("ActionScriptGeneration", "DisplayColumn")};
	}
</#if>
}
</#assign>
<#-- ====================================================================================== -->
<#-- Package declaration and imports -->
/**
 * This is a generated class and is not intended for modfication.  To customize behavior
 * of this value object you may modify the generated sub-class of this class - ${entity.name}.as.
 */

<#if packageMap.get(entity.getName())??>
package ${packageMap.get(entity.getName())}
<#else>
package ${model.name}
</#if>
{
<#list typeHelper.getImports() as imp>
    <#if imp??> 
import ${imp};
    </#if>
</#list>
import flash.events.EventDispatcher;
import flash.net.registerClassAlias;
import flash.net.getClassByAlias;
import com.adobe.fiber.core.model_internal;
import com.adobe.fiber.valueobjects.IPropertyIterator;
import com.adobe.fiber.valueobjects.IValueObject;
import com.adobe.fiber.valueobjects.AvailablePropertyIterator;
import com.adobe.fiber.services.IFiberManagingService;
import mx.binding.utils.ChangeWatcher;
import mx.rpc.AbstractService;

<#-- import async classes if any of the derived properties have service function calls -->
<#if (entity.getDependentOnServices().size() > 0)>
import mx.rpc.IResponder;
import mx.rpc.Responder;
import mx.rpc.events.ResultEvent;
import mx.rpc.events.FaultEvent;
import mx.rpc.AsyncToken;
</#if>

<#-- import events to support internal ChangeWatchers -->
import flash.events.Event;
import mx.events.CollectionEvent;
import mx.events.PropertyChangeEvent;

<#-- style related imports -->
<#if "true" == atLeastOneStyle>
import mx.resources.ResourceManager;
import mx.events.ValidationResultEvent;
import com.adobe.fiber.util.FiberUtils;
</#if>

use namespace model_internal;

${classbody}
}
