<#--*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2008 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and may be covered by U.S. and Foreign Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************-->
 <#--
 #  properties list (no trailing comma)
 #-->
<#macro listProps coll><@compress single_line = true>
    <#list coll.iterator() as coll>"${coll.name}"<#if coll_has_next>, </#if></#list>
</@compress></#macro>

<#assign classbody>
[ExcludeClass]
internal class _${entity.name?cap_first}EntityMetadata extends AbstractEntityMetadata
{
	private static var emptyArray:Array = new Array();

    model_internal static var allProperties:Array = new Array(<@listProps entity.getProperties()/>);
    model_internal static var allAssociationProperties:Array = new Array(<@listProps entity.getAssociationProperties()/>);
    model_internal static var allRequiredProperties:Array = new Array(<@listProps entity.getRequiredProperties()/>);
    model_internal static var allAlwaysAvailableProperties:Array = new Array(<@listProps entity.getAlwaysAvailableProperties()/>);
    model_internal static var guardedProperties:Array = new Array(<@listProps entity.getGuardedProperties()/>);
    model_internal static var dataProperties:Array = new Array(<@listProps entity.getDataProperties()/>);
    model_internal static var derivedProperties:Array = new Array(<@listProps entity.getDerivedProperties()/>);
    model_internal static var collectionProperties:Array = new Array(<@listProps entity.getCollectionProperties()/>);
    model_internal static var collectionBaseMap:Object;    
    model_internal static var entityName:String = "${entity.name}";
    model_internal static var dependentsOnMap:Object;    
    model_internal static var dependedOnServices:Array =  new Array(<@listProps entity.getDependentOnServices()/>);
    
<#-- style vars -->
<#list entity.getProperties().iterator() as prop>                
	<#if prop.getStyle()??>
	model_internal var _${prop.getName()}Style:Style;
	<#-- style validation vars -->  
		<#if (prop.getStyle().getValidations().size() > 0)>
    <#if typeHelper.addImport("com.adobe.fiber.styles.StyleValidator", false) == "_NONE_">
    </#if> 		
	model_internal var _${prop.getName()}IsValid:Boolean;	
	model_internal var _${prop.getName()}ValidationFailureMessages:Array;
	model_internal var _${prop.getName()}StyleValidCacheInitialized:Boolean = false;	
	model_internal var _${prop.getName()}StyleValidator:StyleValidator;
			<#list prop.getStyle().getValidations() as validation>
				<#if validation.getAnnotationGroup("ActionScriptValidator")??>
				    <#if typeHelper.addImport("mx.validators.Validator", false) == "_NONE_">
   				    </#if> 
	model_internal var _${prop.getName()}BuiltInStyleValidator:${validation.getAnnotationItem("ActionScriptValidator", "ValidatorClass")};	
				</#if>
			</#list>
		</#if>
	</#if>           
</#list>    
    
    model_internal var _instance:_Super_${entity.name};
    <#-- static null style --> 
    model_internal static var _nullStyle:Style = new Style();
    
    public function _${entity.name?cap_first}EntityMetadata(${entity.name?uncap_first} : _Super_${entity.name})
    {
        // initialize property maps
        if (model_internal::dependentsOnMap == null)
        {
            // depenents map
            model_internal::dependentsOnMap = new Object();
            <#list entity.getDataProperties().iterator() as prop>
            model_internal::dependentsOnMap["${prop.name}"] = new Array(<@listProps entity.getDependentProperties(prop, true)/>);
            </#list>
                        
            // collection base map
            model_internal::collectionBaseMap = new Object()
            <#list entity.getCollectionProperties().iterator() as prop>
            model_internal::collectionBaseMap["${prop.name}"] = "${util.formatImport(model, packageMap, typeHelper.format(prop.getType().getLeafType()))}";
            </#list>                                                            
        }                        
        
        model_internal::_instance = ${entity.name?uncap_first};     
	<#list entity.getProperties().iterator() as prop>                
		<#if prop.getStyle()??>
			<#if  (prop.getStyle().getValidations().size() > 0)>
		model_internal::_${prop.getName()}StyleValidator = new StyleValidator(model_internal::_instance.model_internal::_doValidationForStyleOf${prop.getName()});
		//model_internal::_${prop.getName()}StyleValidator.source = model_internal::_instance;
		//model_internal::_${prop.getName()}StyleValidator.property = "${prop.getName()}";
				<#list prop.getStyle().getValidations() as validation>
					<#if validation.getAnnotationGroup("ActionScriptValidator")??>
		model_internal::_${prop.getName()}BuiltInStyleValidator = new 	${typeHelper.addImport(validation.getAnnotationItem("ActionScriptValidator", "ValidatorClass"), false)}();
		//model_internal::_${prop.getName()}BuiltInStyleValidator.source = model_internal::_instance;
		//model_internal::_${prop.getName()}BuiltInStyleValidator.property = "${prop.getName()}";
		model_internal::_${prop.getName()}BuiltInStyleValidator.required = false;
						<#list validation.getAnnotationGroup("ActionScriptValidator").entrySet() as entry>
							<#if entry.getKey() != "ValidatorClass">
		model_internal::_${prop.getName()}BuiltInStyleValidator.${entry.getKey()} = ${entry.getValue()};				
							</#if>
						</#list>
					</#if>
				</#list>
			</#if>		
		</#if>           
	</#list>             
    }
    
    override public function getEntityName():String
    {
        return model_internal::entityName;
    }    
    
    override public function getProperties():Array
    {
        return model_internal::allProperties;
    }
    
    override public function getAssociationProperties():Array
    {
        return model_internal::allAssociationProperties;
    }    
    
    override public function getRequiredProperties():Array
    {
         return model_internal::allRequiredProperties;   
    }
    
    override public function getDataProperties():Array
    {
        return model_internal::dataProperties;
    }    
    
    override public function getGuardedProperties():Array
    {
        return model_internal::guardedProperties;
    }     
    
    override public function getUnguardedProperties():Array
    {
        return model_internal::allAlwaysAvailableProperties;
    }        
    
    override public function getDependants(propertyName:String):Array
    {
       if (model_internal::dataProperties.indexOf(propertyName) == -1)
            throw new Error(propertyName + " is not a data property of entity ${entity.name}");  
            
       return model_internal::dependentsOnMap[propertyName] as Array;  
    }
    
    override public function getDependedOnServices():Array
    {
        return model_internal::dependedOnServices;
    }
    
    override public function getCollectionProperties():Array
    {
        return model_internal::collectionProperties;
    }
    
    override public function getCollectionBase(propertyName:String):String
    {
       if (model_internal::collectionProperties.indexOf(propertyName) == -1)
            throw new Error(propertyName + " is not a collection property of entity ${entity.name}");  
            
       return model_internal::collectionBaseMap[propertyName];
    }                
    
     override public function getAvailableProperties():IPropertyIterator
     {
     	return new AvailablePropertyIterator(this); 
     }  
     
     override public function getValue(propertyName:String):*    
     {
         if (model_internal::allProperties.indexOf(propertyName) == -1)
         {
         	throw new Error(propertyName + " does not exist for entity ${entity.name}");
         }     
           
         return model_internal::_instance[propertyName];
     }       
     
     override public function setValue(propertyName:String, value:*):void   
     {
         if (model_internal::dataProperties.indexOf(propertyName) == -1)
         {
              throw new Error(propertyName + " is not a data property of entity ${entity.name}");
         }  
                                                     
         model_internal::_instance[propertyName] = value;
     }     
     
     <#-- getMappedASSociationProperty API -->
     override public function getMappedByProperty(associationProperty:String):String
     {
        switch(associationProperty)
        {
        <#list entity.getDataProperties().iterator() as dataProp>
            <#if dataProp.isAssociation() && dataProp.getMappedByExpression()??>
            case("${dataProp.name}"):
            {
                return "${dataProp.getMappedByExpression()}";
            } 
            </#if>
        </#list>             
            default:
            {
                return null;
            }
        }   
     }
     
     <#-- property length -->
     override public function getPropertyLength(propertyName:String):int
     {
        switch(propertyName)
        {
        <#list entity.getDataProperties().iterator() as dataProp>
            <#if (dataProp.getLength() > 0)>
            case("${dataProp.name}"):
            {
                return ${dataProp.getLength()};
            } 
            </#if>
        </#list>             
            default:
            {
                return 0;
            }
        }  
     }
     
     <#-- General isAvailable property API -->
     override public function isAvailable(propertyName:String):Boolean
     {         
         if (model_internal::allProperties.indexOf(propertyName) == -1)
         {
         	throw new Error(propertyName + " does not exist for entity ${entity.name}");
         }   
                
         if (model_internal::allAlwaysAvailableProperties.indexOf(propertyName) != -1)
         {
         	return true;
         }
         
         switch(propertyName)
         {
            <#list entity.getGuardedProperties().iterator() as prop>
            case("${prop.name}"):
            {
                return is${prop.name?cap_first}Available;
            } 
            </#list>                 
            default:
            {
                return true;
            }
         }                                  
     }  
     
     override public function getIdentityMap():Object
     {
         var returnMap:Object = new Object();
         <#list entity.getIdProperties().iterator() as prop>
         returnMap["${prop.name}"] = model_internal::_instance.${prop.name};
         </#list> 
       
         return returnMap;
     }       
     
	 [Bindable(event="propertyChange")]
	 override public function get invalidConstraints():Array
     {
        if (model_internal::_instance.model_internal::_cacheInitialized_isValid)
        {
            return model_internal::_instance.model_internal::_invalidConstraints;
        }
        else
        {
            // recalculate isValid
            model_internal::_instance.model_internal::_isValid = model_internal::_instance.model_internal::calculateIsValid();
            model_internal::_instance.model_internal::_cacheInitialized_isValid = true;
            return model_internal::_instance.model_internal::_invalidConstraints;        
        }        
     }       
     
	 [Bindable(event="propertyChange")]
	 override public function get validationFailureMessages():Array
     {
        if (model_internal::_instance.model_internal::_cacheInitialized_isValid)
        {
            return model_internal::_instance.model_internal::_validationFailureMessages;
        }
        else
        {
            // recalculate isValid
            model_internal::_instance.model_internal::_isValid = model_internal::_instance.model_internal::calculateIsValid();
            model_internal::_instance.model_internal::_cacheInitialized_isValid = true;
            return model_internal::_instance.model_internal::_validationFailureMessages;        
        }        
     }       
     
     <#-- API for returning invalid constraints that depend on a particular property -->
     override public function getDependantInvalidConstraints(propertyName:String):Array
     {
        var dependants:Array = getDependants(propertyName);
        if (dependants.length == 0)
        {
            return emptyArray;
        }
        
        var currentlyInvalid:Array = invalidConstraints;
        if (currentlyInvalid.length == 0)
        {
            return emptyArray;
        }
            
        var filterFunc:Function = function(element:*, index:int, arr:Array):Boolean
        {
            return dependants.indexOf(element) > -1;     
        }                              
        
        return currentlyInvalid.filter(filterFunc);
     }      
     
    /**
     * isValid
     */
    [Bindable(event="propertyChange")] 
    public function get isValid() : Boolean
    {
        if (model_internal::_instance.model_internal::_cacheInitialized_isValid)
        {
            return model_internal::_instance.model_internal::_isValid;
        }
        else
        {
            // recalculate isValid
            model_internal::_instance.model_internal::_isValid = model_internal::_instance.model_internal::calculateIsValid();
            return model_internal::_instance.model_internal::_isValid;        
        }
    }       
     
<#list entity.getProperties().iterator() as prop>  
	<#if prop.hasGuard()>                 
    <#-- isAvailable getter -->
    [Bindable(event="propertyChange")]   
    public function get is${prop.getName()?cap_first}Available():Boolean
    {
     
        if (model_internal::_instance.model_internal::_isAvailableCache_${prop.name})
        {
            return model_internal::_instance.model_internal::_isAvailable${prop.name?cap_first};
        }     
        else
        {
            // recalculate isAvailable for this property
            model_internal::_instance.model_internal::calculateIsAvailable${prop.name?cap_first}();
            return model_internal::_instance.model_internal::_isAvailable${prop.name?cap_first};
        }                      
     }                      
     <#else>
     <#-- isAvailable getter for properties outside variants always returns true-->   
     [Bindable(event="propertyChange")] 
     public function get is${prop.getName()?cap_first}Available():Boolean
     {
        return true;
     }                       
     </#if>             
                  
</#list>    

<#-- the invalidate dependent on methods called from property setters trigger the recalculation of all 
     properties that depend on the invalidated (set) property either via expression terms or guard terms -->
    /**
     * derived property recalculation
     */
<#list entity.getDataProperties().iterator() as prop>
     <#if (entity.getDependentProperties(prop, true).size() > 0 || entity.getGuardDependentProperties(prop, true).size() > 0
     		|| entity.getStyleDependentProperties(prop, true).size() > 0)>                      
     public function invalidateDependentOn${prop.name?cap_first}():void
     {
     <#if (entity.getDependentProperties(prop, true).size() > 0)>
        <#list entity.getDependentProperties(prop, true) as dependentProp>
            <#-- Make sure that automatic recalculation is not disabled for this property -->
            <#if !((dependentProp.getAnnotationItem("ActionScriptGeneration", "DisableAutoReCalc")?? && dependentProp.getAnnotationItem("ActionScriptGeneration", "DisableAutoReCalc")=="true") || 
                     (entity.getAnnotationItem("ActionScriptGeneration", "DisableAutoReCalc")?? && entity.getAnnotationItem("ActionScriptGeneration", "DisableAutoReCalc")=="true"))>
        if (model_internal::_instance.model_internal::_cacheInitialized_${dependentProp.name} <#if dependentProp.hasGuard()>&& is${dependentProp.name?cap_first}Available</#if>)
        {
            model_internal::_instance.model_internal::calculate${dependentProp.name?cap_first}();
        }            
            </#if>
        </#list>
     </#if>          
     <#if (entity.getGuardDependentProperties(prop, true).size() > 0) >      
     	// availability cache invalidation
        <#list entity.getGuardDependentProperties(prop, true) as dependentProp>      
        if (model_internal::_instance.model_internal::_isAvailableCache_${dependentProp.name})
        {
        	model_internal::_instance.model_internal::calculateIsAvailable${dependentProp.name?cap_first}();
       	}                    
        </#list>    
     </#if>    
     <#if (entity.getStyleDependentProperties(prop, true).size() > 0) >      
     	// style dependance recalculation
        <#list entity.getStyleDependentProperties(prop, true) as dependentProp>      
            <#if (entity.getProperty(dependentProp.getName()).getStyle().getValidations().size() > 0)>
                <#if dependentProp.hasGuard()>
        if (is${dependentProp.name?cap_first}Available)
                </#if>
            model_internal::_instance.model_internal::_doValidationForStyleCacheOf${dependentProp.getName()} = null;
            </#if>      
                <#if dependentProp.hasGuard()>
        if (is${dependentProp.name?cap_first}Available)
                </#if>
            model_internal::_instance.model_internal::_calculate${dependentProp.name?cap_first}Style();           
        </#list>    
     </#if>       
     }
     </#if>                  
</#list>            

     model_internal function fireChangeEvent(propertyName:String, oldValue:Object, newValue:Object):void
     {
     	this.dispatchEvent(mx.events.PropertyChangeEvent.createUpdateEvent(this, propertyName, oldValue, newValue));
     }

<#-- style getters and style validation calculators -->
<#list entity.getProperties().iterator() as prop>                
    [Bindable(event="propertyChange")]   
    public function get ${prop.getName()}Style():Style
    {
	<#if prop.getStyle()??>
		<#-- initialize style if it has not been initialized yet -->
		if (model_internal::_${prop.getName()}Style == null)
		{
			model_internal::_${prop.getName()}Style = new Style();
			<#list prop.getStyle().getMessageAspects().entrySet() as entry>
			model_internal::_${prop.getName()}Style.${entry.getKey()} = new LocalizeableMessage();
			</#list>
			model_internal::_instance.model_internal::_calculate${prop.getName()?cap_first}Style();
		}
		return  model_internal::_${prop.getName()}Style;
	<#else>				
		<#-- return static null style instance -->
		return model_internal::_nullStyle;		
	</#if>
    }      
    
 	<#if prop.getStyle()?? && (prop.getStyle().getValidations().size() > 0)>
    model_internal function set _${prop.getName()}IsValid_der(value:Boolean):void 
    {
        var oldValue:Boolean = model_internal::_${prop.getName()}IsValid;         
        if (oldValue !== value)
        {
        	model_internal::_${prop.getName()}IsValid = value;
        	<#-- Recalculate isValid as it may have changed due to validations of this property changing -->
        	this.dispatchEvent(mx.events.PropertyChangeEvent.createUpdateEvent(this, "${prop.getName()}IsValid", oldValue, value));
        }                             
    }    	
 	
	[Bindable(event="propertyChange")] 	
 	public function get ${prop.getName()}IsValid():Boolean
 	{
 		if (!model_internal::_${prop.getName()}StyleValidCacheInitialized)
 		{
 		   <#if typeHelper.addImport("mx.events.ValidationResultEvent", false) == "_NONE_">
           </#if>
 			var valRes:ValidationResultEvent = model_internal::_${prop.getName()}StyleValidator.validate(model_internal::_instance.${prop.getName()})
            model_internal::_${prop.getName()}IsValid_der = (valRes.results == null);
            <#-- update property level validation failures array -->
            if (valRes.results == null)
                 model_internal::${prop.getName()}ValidationFailureMessages_der = emptyArray;
            else
            {
                 var _valFailures:Array = new Array();
                 for (var a:int = 0 ; a<valRes.results.length ; a++)
                 {
                      _valFailures.push(valRes.results[a].errorMessage);                    
                 }
                 model_internal::${prop.getName()}ValidationFailureMessages_der = _valFailures;
             }   
 	      }
 		
 		return model_internal::_${prop.getName()}IsValid;
 	}
 	
	[Bindable(event="propertyChange")] 	
 	public function get ${prop.getName()}ValidationFailureMessages():Array
 	{
 		if (model_internal::_${prop.getName()}ValidationFailureMessages == null)
 			model_internal::_instance.model_internal::calculateIsValid();
 		
 		return _${prop.getName()}ValidationFailureMessages;
 	} 	
 	
 	model_internal function set ${prop.getName()}ValidationFailureMessages_der(value:Array) : void
    {  
        var oldValue:Array = model_internal::_${prop.getName()}ValidationFailureMessages;
        // avoid firing the event when old and new value are different empty arrays
        
        var needUpdate : Boolean = false;
        
        if (oldValue == null)
            needUpdate = true;
            
        if (!needUpdate && (oldValue !== value && (oldValue.length > 0 || value.length > 0)))
        {
            if (oldValue.length == value.length)
            {
                for (var a:int=0; a < oldValue.length; a++)
                {
                    if (oldValue[a] !== value[a])
                    {
                        needUpdate = true;
                        break;
                    }
                }
            }
            else
            {
                needUpdate = true;
            }
        }
        
        if (needUpdate)
        {
            model_internal::_${prop.getName()}ValidationFailureMessages = value;   
            this.dispatchEvent(mx.events.PropertyChangeEvent.createUpdateEvent(this, "${prop.getName()}ValidationFailureMessages", oldValue, value)); 
            model_internal::_instance.model_internal::isValid_der = model_internal::_instance.model_internal::calculateIsValid();
        }                    
    }  
 	
 	</#if>                             
</#list>      

     /**
     * 
     * @inheritDoc 
     */ 
     override public function getStyle(propertyName:String):IStyle
     {
         switch(propertyName)
         {
            <#list entity.getProperties().iterator() as prop>  
            	<#if prop.getStyle()??>
            case("${prop.name}"):
            {
                return ${prop.getName()}Style;
            } 
            	</#if>
            </#list>                 
            default:
            {
                return null;
            }
         }
     }
     
     /**
     * 
     * @inheritDoc 
     *  
     */  
     override public function getPropertyValidationFailureMessages(propertyName:String):Array
     {
         switch(propertyName)
         {
            <#list entity.getProperties().iterator() as prop>  
            	<#if prop.getStyle()?? && (prop.getStyle().getValidations().size() > 0)>
            case("${prop.name}"):
            {
                return ${prop.getName()}ValidationFailureMessages;
            } 
            	</#if>
            </#list>                 
            default:
            {
                return emptyArray;
            }
         }
     }
    
    <#-- expose derived property recalculators.  Useful when folks have disabled automatic recalcualtion of derived properties -->     
<#list entity.getDerivedProperties() as derivedProp>
   /**
     * Trigger a calculation of derived property "${derivedProp.name}"
     * By default this property is recalculated automatically when any
     * data properties that it depends on changes.
    <#if (entity.getAnnotationItem("ActionScriptGeneration", "DisableAutoReCalc")?? && entity.getAnnotationItem("ActionScriptGeneration", "DisableAutoReCalc")=="true")>
     * The containing model has disabled automatic recalculation for all derived properties of this entity, "${entity.getName()}".
     <#else>
        <#if (derivedProp.getAnnotationItem("ActionScriptGeneration", "DisableAutoReCalc")?? && derivedProp.getAnnotationItem("ActionScriptGeneration", "DisableAutoReCalc")=="true")>
     * The containing model has disabled automatic recalculation for this property.
        </#if>
     </#if> 
     */ 
    public function calculate_${derivedProp.getName()}():void
    {
        model_internal::_instance.model_internal::calculate${derivedProp.name?cap_first}(); 
        <#-- We also need to recalculate all derived properties that depend on this manually recalculated property.  
        We need to do this in dependency chain order otherwise some properties may be recalculated with stale values-->
        <#if (entity.getDependentProperties(derivedProp, true).size() > 0)>
        // calculate dependent derived properties
            <#list entity.getDependentProperties(derivedProp, true) as dependantDerivedProp>
        if (model_internal::_instance.model_internal::_cacheInitialized_${dependantDerivedProp.name})
        {
            model_internal::_instance.model_internal::calculate${dependantDerivedProp.name?cap_first}();
        }   
            </#list>
        </#if>
        <#-- style dependent props -->
        <#if (entity.getStyleDependentProperties(derivedProp, true).size() > 0)>
        // calculate dependent derived properties
            <#list entity.getStyleDependentProperties(derivedProp, true) as styleDepProp>
                <#if (entity.getProperty(styleDepProp.getName()).getStyle().getValidations().size() > 0)>
        model_internal::_instance.model_internal::_doValidationForStyleCacheOf${styleDepProp.getName()} = null;
                </#if>      
        model_internal::_instance.model_internal::_calculate${styleDepProp.name?cap_first}Style();      
            </#list>
        </#if>     
        <#-- availability dependence -->
        <#if (entity.getGuardDependentProperties(derivedProp, true).size() > 0) >      
        // availability recalculation
        <#list entity.getGuardDependentProperties(derivedProp, true) as availDepProp>      
        if (model_internal::_instance.model_internal::_isAvailableCache_${availDepProp.name})
        {
            model_internal::_instance.model_internal::calculateIsAvailable${availDepProp.name?cap_first}();
        }                    
        </#list>    
     </#if>       
    }
</#list>  
}
</#assign>
<#-- ====================================================================================== -->
/**
 * This is a generated class and is not intended for modfication.  
 */
<#if packageMap.get(entity.getName())??>
package ${packageMap.get(entity.getName())}
<#else>
package ${model.name}
</#if>
{
<#list typeHelper.getImports() as imp>
    <#if imp??> 
import ${imp};
    </#if>
</#list>
import com.adobe.fiber.core.model_internal;
import com.adobe.fiber.styles.IStyle;
import com.adobe.fiber.styles.Style;
import com.adobe.fiber.styles.LocalizeableMessage;
import com.adobe.fiber.valueobjects.IModelType;
import com.adobe.fiber.valueobjects.IPropertyIterator;
import com.adobe.fiber.valueobjects.AvailablePropertyIterator;
import com.adobe.fiber.valueobjects.AbstractEntityMetadata;
import mx.events.PropertyChangeEvent;

use namespace model_internal;

${classbody}
}
