<#--*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2008 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and may be covered by U.S. and Foreign Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************-->
 <#--
 #  properties list with types (no trailing comma)
 #-->
<#macro listIdPropsWithTypes coll><@compress single_line = true>
    <#list coll.iterator() as coll>${coll.getName()}:${typeHelper.format(coll.getType())}<#if coll_has_next>, </#if></#list>
</@compress></#macro>
 <#--
 #  properties list into a map (no trailing comma)
 #-->
<#macro listIdPropsForMap coll><@compress single_line = true>
    <#list coll.iterator() as coll>"${coll.getKey()}":${termFormatter.format(coll.getValue().getExpressionTerm())}<#if coll_has_next>, </#if></#list>
</@compress></#macro>
 <#-- Body -->
<#assign classbody>   
[ExcludeClass]
internal class _Super_${entity.name?cap_first}Service extends DataServiceWrapper 
{  
        <#if entity.getAnnotationItem("ServerProperties", "Destination")??>
            <#assign destinationName>${entity.getAnnotationItem('ServerProperties', 'Destination')}</#assign>
        <#else>
    <#-- Naming Convention for FiberAssembler backed, DestinationManager configured destinations comes into play here -->
            <#if model.getAnnotationItem("ModelName", "prefix")??>
                <#assign destinationName>${model.getAnnotationItem('ModelName', 'prefix')}_${model.name}.${entity.name}</#assign>
            <#else>
                 <#assign destinationName>${model.name}.${entity.name}</#assign>
            </#if>       
        </#if>     
    public function _Super_${entity.name?cap_first}Service(destination:String="${destinationName}")
    {
        super(destination);        
        <#-- create unique data store if that annotation is set or the destination has no associating entities-->
        <#if entity.getAnnotationItem("ActionScriptGeneration", "UniqueDataStore")??>
            <#if entity.getAnnotationItem("ActionScriptGeneration", "UniqueDataStore")=="true" >
        _serviceControl.dataStore = new DataStore("${destinationName}", true);
            </#if>
        </#if>
<#-- Channel Set and Destination -->
<#-- Channel Set specified at the service level takes precedence over a reference to a model level one-->
<#if entity.getAnnotationGroup("ASChannelSet")??>
        var cs:ChannelSet = new ChannelSet();
        <#if typeHelper.addImport("mx.messaging.ChannelSet;", false) == "_NONE_">
        </#if>
    <#list entity.getAnnotationGroup("ASChannelSet").entrySet() as channelEntry>
        <#assign firstComma = channelEntry.getValue()?index_of(",")/>
        <#assign secondComma = (channelEntry.getValue()?substring(firstComma+1))?index_of(",") + firstComma + 1/>
        <#assign channelName = channelEntry.getValue()?substring(firstComma+1, secondComma)/>
        <#if channelName == "null">
        cs.addChannel(new ${channelEntry.getValue()?substring(0, firstComma)}(${channelName}, "${channelEntry.getValue()?substring(secondComma+1)}"));
        <#else>
        cs.addChannel(new ${channelEntry.getValue()?substring(0, firstComma)}("${channelName}", "${channelEntry.getValue()?substring(secondComma+1)}"));        
        </#if>
        <#if typeHelper.addImport("mx.messaging.channels." + channelEntry.getValue()?substring(0, firstComma) + ";", false) == "_NONE_">
        </#if>      
    </#list>
        _serviceControl.channelSet = cs;
<#-- Channel Set at model level reference-->
<#elseif entity.getAnnotationItem("ActionScriptGeneration", "ChannelSet")??>
	<#assign channelSetAnnotationName = "ASChannelSet-" + entity.getAnnotationItem("ActionScriptGeneration", "ChannelSet")/>
		<#-- TODO: Exception if channel set is specified in entity but not at model level.
		This should eventually be done by the Annotation Validator for ASGeneration -->
		var cs:ChannelSet = new ChannelSet();
		<#if typeHelper.addImport("mx.messaging.ChannelSet;", false) == "_NONE_">
		</#if>
		<#list model.getAnnotationGroup(channelSetAnnotationName).entrySet() as channelEntry>
			<#assign firstComma = channelEntry.getValue()?index_of(",")/>
			<#assign secondComma = (channelEntry.getValue()?substring(firstComma+1))?index_of(",") + firstComma + 1/>
			<#assign channelName = channelEntry.getValue()?substring(firstComma+1, secondComma)/>
			<#if channelName == "null">
	   cs.addChannel(new ${channelEntry.getValue()?substring(0, firstComma)}(${channelName}, "${channelEntry.getValue()?substring(secondComma+1)}"));
			<#else>
		cs.addChannel(new ${channelEntry.getValue()?substring(0, firstComma)}("${channelName}", "${channelEntry.getValue()?substring(secondComma+1)}"));
			</#if>
			<#if typeHelper.addImport("mx.messaging.channels." + channelEntry.getValue()?substring(0, firstComma) + ";", false) == "_NONE_">
			</#if>		
		</#list>
		_serviceControl.channelSet = cs;
</#if>                
        model_internal::initialize();          
        
       // initialize RemoteClass alias for entity managed by this service
       <#if packageMap.get(entity.getName())??>
        ${packageMap.get(entity.getName())}.${entity.getName()}._initRemoteClassAlias();
       <#else>
        ${model.name}.${entity.getName()}._initRemoteClassAlias();
        </#if>                                              
    } 
    
   /**
	* This method is a generated wrapper used to call the 'createItem' operation of the underlying DataService. It returns an ItemReference whose 
	* result property will be populated with the result of the operation when the server response is received. 
	* To use this result from MXML code, define a CallResponder component and assign its token property to this method's return value. 
	* You can then bind to CallResponder.lastResult or listen for the CallResponder.result or fault events.
    *
    * @see mx.data.ItemReference
    * @see mx.rpc.CallResponder 
    * @see mx.data.DataService
    *
    * @return an ItemReference whose result property will be populated with the result of the operation when the server response is received.
	*/      
    public function create${entity.name}(${entity.name?uncap_first}:${typeHelper.format(entity)}):ItemReference
    {
        var token:ItemReference = _serviceControl.createItem(${entity.name?uncap_first});
        return token;    
    }
    
   /**
	* This method is a generated wrapper used to call the 'deleteItem' operation of the underlying DataService. It returns an AsyncToken whose 
	* result property will be populated with the result of the operation when the server response is received. 
	* To use this result from MXML code, define a CallResponder component and assign its token property to this method's return value. 
	* You can then bind to CallResponder.lastResult or listen for the CallResponder.result or fault events.
    *
    * @see mx.rpc.AsyncToken
    * @see mx.rpc.CallResponder 
    * @see mx.data.DataService
    *
    * @return an AsyncToken whose result property will be populated with the result of the operation when the server response is received.
	*/     
    public function delete${entity.name}(${entity.name?uncap_first}:${typeHelper.format(entity)}):AsyncToken
    {
        var token:AsyncToken = _serviceControl.deleteItem(${entity.name?uncap_first});
        return token;                 
    }
    
   /**
    *  Updates the passed-in item and commits pending changes for that item only.  
    *  Returns the token from the update invocation.
    *
    *  @param ${entity.name?uncap_first} item for which changes are committed
    *
    *  @param origItem version of the item before changes were made.  This version is used for 
    *  conflict detection.  May be null.
    * 
    *  @param changes array of property names that have been updated.  Used for performance
    *  improvement.  May be null. 
    *
    *  @return token from the update invocation.
    *
    *  @see mx.data.DataService
    */
    public function update${entity.name}(${entity.name?uncap_first}:${typeHelper.format(entity)}, origItem:Object=null, changes:Array=null):AsyncToken
    {
        // perform the update
        var token:AsyncToken = _serviceControl.updateItem(${entity.name?uncap_first}, origItem, changes);
        
        // perform the commit
        var commitArray:Array = new Array();
        commitArray.push(${entity.name?uncap_first});
        _serviceControl.commit(commitArray);
        
        return token;
    }      
    
  /**
	* This method is a generated wrapper used to call the 'getItem' operation of the underlying DataService. It returns an ItemReference whose 
	* result property will be populated with the result of the operation when the server response is received. 
	* To use this result from MXML code, define a CallResponder component and assign its token property to this method's return value. 
	* You can then bind to CallResponder.lastResult or listen for the CallResponder.result or fault events.
    *
    * @see mx.data.ItemReference
    * @see mx.rpc.CallResponder 
    * @see mx.data.DataService
    *
    * @return an ItemReference whose result property will be populated with the result of the operation when the server response is received.
	*/   
    public function get${entity.name}(<@listIdPropsWithTypes entity.getIdProperties()/>):ItemReference
    {
        var token:ItemReference = _serviceControl.getItem({<@listIdPropsForMap util.getPrimitiveIdPropsInfo(model, entity, true).entrySet()/>});        
        return token;            
    }    
    
    // FILL FUNCTIONS
    <#if entity.getAnnotationItem("ServerProperties", "CustomDestinationFlag")?? && entity.getAnnotationItem("ServerProperties", "CustomDestinationFlag")=="true">
        <#list entity.getFilters(declaredFilterKind) as filter>
            <#-- Support the DefaultFill annotation for LCDS, AbstractAssembler extended custom assemblers (no explicit fill declarations -->
            <#if filter.getAnnotationItem("ActionScriptGeneration", "DefaultFill")?? && filter.getAnnotationItem("ActionScriptGeneration", "DefaultFill")=="true">
   /**
    * This method is a generated wrapper used to call the fill function of the underlying DataService.  The variable length "args" argument allows
    * you to specify all of the parameters required by your server side fill operation.  This function returns an AsyncToken whose 
    * result property will be populated with the result of the operation when the server response is received. 
    * To use this result from MXML code, define a CallResponder component and assign its token property to this method's return value. 
    * You can then bind to CallResponder.lastResult or listen for the CallResponder.result or fault events.
    *
    * @see mx.rpc.AsyncToken
    * @see mx.rpc.CallResponder 
    *
    * @return an AsyncToken whose result property will be populated with the result of the operation when the server response is received.
    */   
    public function ${filter.getName()}(...args:Array):AsyncToken
    {
        var ac:ArrayCollection = new ArrayCollection();
        var paramsArray:Array = new Array();
        paramsArray.push(ac);
        paramsArray = paramsArray.concat(args);
        var token:AsyncToken = _serviceControl.fill.apply(this, paramsArray);
        token.resultAC = ac;
        return token;
    }
            <#else>
   /**
	* This method is a generated wrapper used to call the '${filter.getName()}' fill operation. It returns an AsyncToken whose 
	* result property will be populated with the result of the operation when the server response is received. 
	* To use this result from MXML code, define a CallResponder component and assign its token property to this method's return value. 
	* You can then bind to CallResponder.lastResult or listen for the CallResponder.result or fault events.
    *
    * @see mx.rpc.AsyncToken
    * @see mx.rpc.CallResponder 
    *
    * @return an AsyncToken whose result property will be populated with the result of the operation when the server response is received.
	*/       
    public function ${filter.getName()}${typeHelper.format(filter.getSignature())}:AsyncToken
    {
        var ac:ArrayCollection = new ArrayCollection(); 
        <#if (filter.getSignature().getArity() > 0)>       
        var token:AsyncToken= _serviceControl.fill(ac, ${util.getServiceArgs(filter.getSignature(), false, null)});                                 
        <#else>
        var token:AsyncToken= _serviceControl.fill(ac);                          
        </#if>                
        token.resultAC = ac;
        return token;          
    }
            </#if>
        </#list>  
    <#else>
        <#list entity.getFilters() as filter>
  /**
	* This method is a generated wrapper used to call the '${filter.getName()}' fill operation. It returns an AsyncToken whose 
	* result property will be populated with the result of the operation when the server response is received. 
	* To use this result from MXML code, define a CallResponder component and assign its token property to this method's return value. 
	* You can then bind to CallResponder.lastResult or listen for the CallResponder.result or fault events.
	*
	* @see mx.rpc.AsyncToken
	* @see mx.rpc.CallResponder 
	*
	* @return an AsyncToken whose result property will be populated with the result of the operation when the server response is received.
    */   
    public function ${filter.getName()}${typeHelper.format(filter.getSignature())}:AsyncToken
    {
        var ac:ArrayCollection = new ArrayCollection();
        <#assign fillParams = ""/>
        <#if (filter.getSignature().getArity() > 0)>
            <#assign fillParams>, ${util.getServiceArgs(filter.getSignature(), false, null)}</#assign>
        </#if>
        <#if clientValidator.getPropertiesInSpecifier(filter)?? && (0 < clientValidator.getPropertiesInSpecifier(filter).size())>
            <#if typeHelper.addImport("mx.data.PropertySpecifier", false) == "_NONE_"></#if>
        var propertySpecifier:PropertySpecifier = new PropertySpecifier(null, PropertySpecifier.INCLUDE_LIST, [${util.formatStringList(clientValidator.getPropertiesInSpecifier(filter), ", ", true)}]);
        var token:AsyncToken = _serviceControl.fillSubset(ac, propertySpecifier, "${filter.getName()}"${fillParams});
        <#else>
        var token:AsyncToken = _serviceControl.fill(ac, "${filter.getName()}"${fillParams});
        </#if>
        token.resultAC = ac;
        return token;
    }
    
        </#list>    
    </#if>                        
}
</#assign>
<#-- ====================================================================================== -->
/**
 * This is a generated class and is not intended for modfication.  To customize behavior
 * of this service wrapper you may modify the generated sub-class of this class - ${entity.name?cap_first}Service.as.
 */
 <#if packageMap.get(entity.getName() + "__ImplicitService")??>
package ${packageMap.get(entity.getName() + "__ImplicitService")}
<#elseif packageMap.get(entity.getName())??>
package ${packageMap.get(entity.getName())}
<#else>
package ${model.name}
</#if>
{
import mx.collections.ArrayCollection;
import mx.data.DataService;
<#if (entity.getAnnotationItem("ActionScriptGeneration", "UniqueDataStore")?? && entity.getAnnotationItem("ActionScriptGeneration", "UniqueDataStore")=="true" ) || 
            (model.getAssociatingEntities(entity, false, false).size() == 0 && entity.getAssociatedEntities(false, false).size() == 0)>
import mx.data.DataStore;
</#if>
import mx.data.ItemReference;
import mx.rpc.AsyncToken;
import com.adobe.fiber.core.model_internal;
import com.adobe.fiber.services.wrapper.DataServiceWrapper;

<#list typeHelper.getImports() as imp>
    <#if imp??> 
import ${imp}
    </#if>
</#list>

${classbody}
}
