<#--*************************************************************************
 *
 * ADOBE CONFIDENTIAL
 * __________________
 *
 *  Copyright 2008 Adobe Systems Incorporated
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Adobe Systems Incorporated and its suppliers,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its
 * suppliers and may be covered by U.S. and Foreign Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Adobe Systems Incorporated.
 **************************************************************************-->
<#-- Constant to enable freemarker scalar to freemarker scalar comparison -->
<#assign isYes>
yes
</#assign>
<#--
 #  properties list (no trailing comma)
 #-->
<#macro listIdProps coll><@compress single_line = true>
    <#list coll.iterator() as coll>${coll.name}<#if coll_has_next>,</#if></#list>
</@compress></#macro>
 <#--
 #  parse operation param list
 #-->
<#macro formParamList annotValue><@compress single_line = true>
    <#if (annotValue?index_of("(") > 0)>
${annotValue?substring(annotValue?index_of("(")+1, annotValue?index_of(")"))}
    <#else>
item
    </#if>    
</@compress></#macro>
 <#--
 #  parse operation 
 #-->
<#macro parseop annotValue><@compress single_line = true>
    <#if (annotValue?index_of("(") > 0)>
${annotValue?substring(0, annotValue?index_of("("))}
    <#else>
${annotValue}
    </#if>    
</@compress></#macro>
<#--
 #  Macro for getting resultType or resultElementType for operations
 #-->
<#macro getResultType function>
    <#if function.getSignature().getReturnType().isCollectionType()>    	
		 operation.resultElementType = ${typeHelper.format(function.getSignature().getReturnType().getLeafType())};
    <#else>
    	<#if function.getSignature().getReturnType().getExpression() == "opaque" && function.getAnnotationItem("OpaqueClarification", "return-type")??>
    	 operation.resultType = ${typeHelper.format(function.getSignature().getReturnType(), function.getAnnotationItem("OpaqueClarification", "return-type"))}; 	 
		<#elseif function.getSignature().getReturnType().getExpression() != "void">
		 operation.resultType = ${typeHelper.format(function.getSignature().getReturnType())}; 		 
		</#if>   
    </#if>
    <#if typeHelper.clearOpaqueClarification()></#if>
</#macro>
<#-- Channel Set and Destination -->
<#assign channelSetAndDestination>
<#-- Channel Set specified at the service level takes precedence over a reference to a model level one-->
<#if service.getAnnotationGroup("ASChannelSet")??>
		var cs:ChannelSet = new ChannelSet();
		<#if typeHelper.addImport("mx.messaging.ChannelSet;", false) == "_NONE_">
		</#if>
	<#list service.getAnnotationGroup("ASChannelSet").entrySet() as channelEntry>
		<#assign firstComma = channelEntry.getValue()?index_of(",")/>
		<#assign secondComma = (channelEntry.getValue()?substring(firstComma+1))?index_of(",") + firstComma + 1/>
		<#assign channelName = channelEntry.getValue()?substring(firstComma+1, secondComma)/>
		<#if channelName == "null">
		cs.addChannel(new ${channelEntry.getValue()?substring(0, firstComma)}(${channelName}, "${channelEntry.getValue()?substring(secondComma+1)}"));
		<#else>
		cs.addChannel(new ${channelEntry.getValue()?substring(0, firstComma)}("${channelName}", "${channelEntry.getValue()?substring(secondComma+1)}"));		
		</#if>
		<#if typeHelper.addImport("mx.messaging.channels." + channelEntry.getValue()?substring(0, firstComma) + ";", false) == "_NONE_">
		</#if>		
	</#list>
		_serviceControl.channelSet = cs;
<#-- Channel Set at model level reference-->
<#elseif service.getAnnotationItem("ActionScriptGeneration", "ChannelSet")??>
<#assign channelSetAnnotationName = "ASChannelSet-" + service.getAnnotationItem("ActionScriptGeneration", "ChannelSet")/>
	<#-- TODO: Exception if channel set is specified in service but not at model level.
	This should eventually be done by the Annotation Validator for ASGeneration -->
		var cs:ChannelSet = new ChannelSet();
		<#if typeHelper.addImport("mx.messaging.ChannelSet;", false) == "_NONE_">
		</#if>
	<#list model.getAnnotationGroup(channelSetAnnotationName).entrySet() as channelEntry>
		<#assign firstComma = channelEntry.getValue()?index_of(",")/>
		<#assign secondComma = (channelEntry.getValue()?substring(firstComma+1))?index_of(",") + firstComma + 1/>
		<#assign channelName = channelEntry.getValue()?substring(firstComma+1, secondComma)/>
		<#if channelName == "null">
		cs.addChannel(new ${channelEntry.getValue()?substring(0, firstComma)}(${channelName}, "${channelEntry.getValue()?substring(secondComma+1)}"));
		<#else>
		cs.addChannel(new ${channelEntry.getValue()?substring(0, firstComma)}("${channelName}", "${channelEntry.getValue()?substring(secondComma+1)}"));		
		</#if>
		<#if typeHelper.addImport("mx.messaging.channels." + channelEntry.getValue()?substring(0, firstComma) + ";", false) == "_NONE_">
		</#if>		
	</#list>
		_serviceControl.channelSet = cs;
</#if>
<#-- Add destination if it is set -->
<#if service.getAnnotationItem("ActionScriptGeneration", "Destination")??>
		_serviceControl.destination = "${service.getAnnotationItem("ActionScriptGeneration", "Destination")}";
<#-- if this is an LCES RemoteObject, initialize destination name to service name -->
<#elseif serviceType == "RemoteObject" && service.getAnnotationItem("ActionScriptGeneration", "LCES-Service")??>
		_serviceControl.destination = "${service.getName()}";
</#if>
</#assign>
<#-- Data Manager Variables if Any--> 
<#assign dmvars>
    <#if managedEntities??>
        <#list managedEntities as entity>
    private var _${entity.getName()?uncap_first}RPCDataManager : RPCDataManager;         
        </#list>
    private var managersArray : Array = new Array();
        
    <#-- Data Manager Constants if Any-->   
        <#list managedEntities as entity>
    public const DATA_MANAGER_${entity.getName()?upper_case} : String = "${entity.getName()}";         
        </#list>
        
    public function getDataManager(dataManagerName:String) : RPCDataManager
    {
        switch (dataManagerName)
        {
        <#list managedEntities as entity>
             case (DATA_MANAGER_${entity.getName()?upper_case}):
                return _${entity.getName()?uncap_first}RPCDataManager;      
        </#list>        
            default:
                return null;
        }
    }
    
    /**
     * Commit all of the pending changes for this DataService, as well as all of the pending changes of all DataServices
     * sharing the same DataStore.  By default, a DataService shares the same DataStore with other DataServices if they have 
     * managed association properties and share the same set of channels. 
     *
     * @see mx.data.DataManager
     * @see mx.data.DataStore
     *
     * @param itemsOrCollections:Array This is an optional parameter which defaults to null when
     *  you want to commit all pending changes.  If you want to commit a subset of the pending
     *  changes use this argument to specify a list of managed ListCollectionView instances
     *  and/or managed items.  ListCollectionView objects are most typically ArrayCollections
     *  you have provided to your fill method.  The items appropriate for this method are
     *  any managed version of the item.  These are any items you retrieve from getItem, createItem
     *  or using the getItemAt method from a managed collection.  Only changes for the
     *  items defined by any of the values in this array will be committed.
     *
     * @param cascadeCommit if true, also commit changes made to any associated
     *  items supplied in this list.
     *
     *  @return AsyncToken that is returned in <code>call</code> property of
     *  either the <code>ResultEvent.RESULT</code> or in the
     *  <code>FaultEvent.FAULT</code>.
     *  Custom data can be attached to this object and inspected later
     *  during the event handling phase.  If no changes have been made
     *  to the relevant items, null is returned instead of an AsyncToken.
     */
    public function commit(itemsOrCollections:Array=null, cascadeCommit:Boolean=false):AsyncToken
    {
    	return _${managedEntities.toArray()[0].getName()?uncap_first}RPCDataManager.dataStore.commit(itemsOrCollections, cascadeCommit);
    }
    
    /**
     *  Reverts all pending (uncommitted) changes for this DataService, as well as all of the pending changes of all DataServics
     *  sharing the same DataStore.  By default, a DataService shares the same DataStore with other DataServices if they have 
     * managed association properties and share the same set of channels. 
     *
     * @see mx.data.DataManager
     * @see mx.data.DataStore
     *
     *  @return true if any changes were reverted.
     *  
     */
    public function revertChanges():Boolean
    {
        return _${managedEntities.toArray()[0].getName()?uncap_first}RPCDataManager.dataStore.revertChanges();
    }    
    </#if> 
</#assign>
<#-- Initialize Data Managers if Any-->  
<#assign dminits>              
    <#if managedEntities??>
        var managedAssociation : ManagedAssociation;
    	var managedAssocsArray : Array;
        <#list managedEntities as entity>
<#assign dmvar>_${entity.getName()?uncap_first}RPCDataManager</#assign>  
        // initialize ${entity.getName()} data manager     
        <#if entity.getAnnotationItem("DataManagement", "dataManager")??>
        ${dmvar} = new ${typeHelper.addImport(entity.getAnnotationItem("DataManagement", "dataManager"), true)}();
        <#else>
        ${dmvar} = new RPCDataManager();        
        </#if>
        managersArray.push(${dmvar});
        
        managedAssocsArray = new Array();
        
        <#assign hasAssocs>no</#assign>
        ${dmvar}.destination = "${entity.getName()?uncap_first}RPCDataManager";
        ${dmvar}.service = _serviceControl;        
        ${dmvar}.identities =  "<@listIdProps entity.getIdProperties()/>";      
        ${dmvar}.itemClass = ${typeHelper.addImport(entity.getName(), true)}; 
        <#if entity.getAnnotationItem("DataManagement", "dataServiceAdapter")??>
        ${dmvar}.adapter = new ${typeHelper.addImport(entity.getAnnotationItem("DataManagement", "dataServiceAdapter"), true)}();
        </#if>
        
        <#-- Initialize Data Management Associations if Any-->          
        	<#list entity.getAssociationProperties() as assocProperty>
        		<#if assocProperty.isData()>             			
        			<#if assocProperty.getType().isCollectionType()>
        				<#assign relatedDest>${assocProperty.getType().getLeafType().getExpression()?uncap_first}RPCDataManager</#assign>
        				<#assign assocEntity>${assocProperty.getType().getLeafType().getExpression()}</#assign>
        			<#else>
        				<#assign relatedDest>${assocProperty.getType().getExpression()?uncap_first}RPCDataManager</#assign>
        				<#assign assocEntity>${assocProperty.getType().getExpression()}</#assign>       				        			
        			</#if>
        			<#if managedEntities.contains(model.getEntity(assocEntity))>
        				<#assign hasAssocs>yes</#assign>
        managedAssociation = new ManagedAssociation();
        managedAssociation.property = "${assocProperty.getName()}";
        managedAssociation.type = "${assocProperty.getAssociationCardinality().toString()}";
        managedAssociation.destination = "${relatedDest}";
        managedAssocsArray.push(managedAssociation);
        			</#if>     			    			     				
        		</#if>  
        		
        	</#list>
        	<#if hasAssocs == "yes">
        ${dmvar}.associations = managedAssocsArray;
        	</#if>        	        
                   
        </#list>
    </#if> 
</#assign> 
<#-- Initialize Data Manager Operations if Any-->  
<#assign dmops>
    <#if functionMap??>
        var dmOperation : ManagedOperation;
        var dmQuery : ManagedQuery;
         
        <#list functionMap.entrySet() as functionMapEntry>       
        <#-- support for remote function mapping -->
		<#if functionMapEntry.getValue().getFunction().getAnnotationItem("ActionScriptGeneration", "remoteFunction")??>
			<#assign remoteFunctionName>${functionMapEntry.getValue().getFunction().getAnnotationItem("ActionScriptGeneration", "remoteFunction")}</#assign>
		<#else>
			<#assign remoteFunctionName>${functionMapEntry.getValue().getFunction().getName()}</#assign>
		</#if>                
        <#assign dmvar>_${functionMapEntry.getValue().getEntity()?uncap_first}RPCDataManager</#assign>  
            <#if functionMapEntry.getValue().getFunctionType() == "fillFunc" || functionMapEntry.getValue().getFunctionType() == "findItem">
 <#-- Fill Func-->         
        dmQuery = new ManagedQuery("${remoteFunctionName}");
                <#if functionMapEntry.getValue().getPropertySpecifier()??>
        dmQuery.propertySpecifier = "${functionMapEntry.getValue().getPropertySpecifier()}";
                </#if>  
                <#if functionMapEntry.getValue().getCountOperation()??>
        dmQuery.countOperation = "${functionMapEntry.getValue().getCountOperation()}";
                </#if> 
                <#if functionMapEntry.getValue().isPagingEnabled()>
        dmQuery.pagingEnabled = true;
                </#if>    
                <#if functionMapEntry.getValue().isPositionalPagingParams()>
        dmQuery.positionalPagingParameters = true;
                </#if>                   
                <#if functionMapEntry.getValue().getPageSize()??>
        dmQuery.pageSize = ${functionMapEntry.getValue().getPageSize()};
                </#if>                  
                <#if functionMapEntry.getValue().getAddItemToCollectionOperation()??>
        dmQuery.addItemToCollectionOperation = "${functionMapEntry.getValue().getAddItemToCollectionOperation()}";
                </#if>   
                <#if functionMapEntry.getValue().getRemoveItemFromCollectionOperation()??>
        dmQuery.removeItemFromCollectionOperation = "${functionMapEntry.getValue().getRemoveItemFromCollectionOperation()}";
                </#if>   
                <#if functionMapEntry.getValue().getParameters()??>				
        dmQuery.parameters = "${functionMapEntry.getValue().getParameters()}";
                </#if>                     
                <#if functionMapEntry.getValue().getFunctionType() == "findItem">
		dmQuery.type="findItem";                
                </#if>                                                                           
        ${dmvar}.addManagedOperation(dmQuery);                 

            <#else>
 <#-- Create, Update, Delete, GetItem Func-->
        dmOperation = new ManagedOperation("${remoteFunctionName}", "${functionMapEntry.getValue().getFunctionType()}");
         <#-- A parameterless getItem function is handled differently because a list of the entities id properties needs to be created here-->
        		<#if ((functionMapEntry.getValue().getFunctionType() == "get") && (functionMapEntry.getValue().getAnnotValue()?index_of("(") == -1))>
        dmOperation.parameters = "<@listIdProps model.getEntity(functionMapEntry.getValue().getEntity()).getIdProperties()/>";
        		<#else>					     		
        dmOperation.parameters = "<@formParamList functionMapEntry.getValue().getAnnotValue()/>";
        		</#if>
        ${dmvar}.addManagedOperation(dmOperation);     
            
            </#if>                      
        </#list>
        _serviceControl.managers = managersArray;
    </#if>
</#assign>
<#-- Headers -->
<#if serviceType == "RemoteObject">
<#assign remoteObjectHeader>
[ExcludeClass]
internal class _Super_${service.name?cap_first} extends RemoteObjectServiceWrapper
{      
${dmvars}       
    // Constructor
    public function _Super_${service.name?cap_first}()
    {
        // initialize service control
        _serviceControl = new RemoteObject(); 
        
<#if (service.getFunctions().size() > 0)>
        var operations:Object = new Object();
        var operation:Operation;         
         
    <#list service.getFunctions() as function>     
        <#-- support for remote function mapping -->
    	<#if function.getAnnotationItem("ActionScriptGeneration", "remoteFunction")??>
			<#assign remoteFunctionName>${function.getAnnotationItem("ActionScriptGeneration", "remoteFunction")}</#assign>
		<#else>
			<#assign remoteFunctionName>${function.getName()}</#assign>
		</#if>        
        operation = new Operation(null, "${remoteFunctionName}");
         <@getResultType function/>
         <#if service.getAnnotationItem("ActionScriptGeneration", "LCES-Service")?? && function.getAnnotationItem("return-type", "SingleValuedMap")??>
        operation.properties = new Object();
		operation.properties["singleValueResult"] = true;
		 </#if>
        operations["${remoteFunctionName}"] = operation;
         <#-- for LCES Services, add a parallel asynchronous invocation function -->
         <#if service.getAnnotationItem("ActionScriptGeneration", "LCES-Service")??>
         
        // asynchronous operation counterpart
        operation = new Operation(null, "${remoteFunctionName}_Async");
        operation.resultType = ${typeHelper.addImport("mx.rpc.livecycle.JobId", true)};
        operations["${remoteFunctionName}_Async"] = operation;
         </#if>
         
         <#-- initialize RemoteClass alias for all entities returned by functions of this service -->
        <#if function.getSignature().getReturnType().isEntity()>
            <#if packageMap.get(function.getSignature().getReturnType().getName())??>
     ${packageMap.get(function.getSignature().getReturnType().getName())}.${function.getSignature().getReturnType().getName()}._initRemoteClassAlias();
            <#else>
     ${model.name}.${function.getSignature().getReturnType().getName()}._initRemoteClassAlias();
            </#if>
        <#else>
            <#if function.getSignature().getReturnType().isCollectionType() && function.getSignature().getReturnType().getElementType().isEntity()>
                <#if packageMap.get(function.getSignature().getReturnType().getElementType().getName())??>
     ${packageMap.get(function.getSignature().getReturnType().getElementType().getName())}.${function.getSignature().getReturnType().getElementType().getName()}._initRemoteClassAlias();
                <#else>
     ${model.name}.${function.getSignature().getReturnType().getElementType().getName()}._initRemoteClassAlias();
                </#if>                
            </#if>   
        </#if>
    </#list>
    
        _serviceControl.operations = operations;   
         <#-- add convert result handled -->
		_serviceControl.convertResultHandler = TypeUtility.convertResultHandler;
		 <#-- if CF RemoteObject add special convert parameters handler for the case where CF assumes a single Object parameter implies a map -->
    <#if service.getAnnotationItem("ServiceTypeGroup", "ServiceBackendType")?? && service.getAnnotationItem("ServiceTypeGroup", "ServiceBackendType")== "ColdFusion" >
    	_serviceControl.convertParametersHandler = TypeUtility.convertCFAMFParametersHandler;
    </#if>
    <#-- To allow RemoteObject service with non-Blaze/LCDS backend, support source and endpoint annotations-->
    <#if service.getAnnotationItem("ActionScriptGeneration", "Source")??>		 
        _serviceControl.source = "${service.getAnnotationItem("ActionScriptGeneration", "Source")}";
    </#if>
    <#if service.getAnnotationItem("ActionScriptGeneration", "Endpoint")??>        
        _serviceControl.endpoint = "${service.getAnnotationItem("ActionScriptGeneration", "Endpoint")}";
    </#if>
</#if>               
${channelSetAndDestination}        
${dminits}    
${dmops}                      
         model_internal::initialize();
    }
</#assign>
</#if>
<#if serviceType == "HTTPMultiService">
<#assign httpMultiHeader>
[ExcludeClass]
internal class _Super_${service.name?cap_first} extends HTTPServiceWrapper
{      
<#list serializerMap.entrySet() as entry>
    private static var ${entry.getValue()}:${util.unqualify(entry.getKey())} = new ${util.unqualify(entry.getKey())}();
</#list>
${dmvars}       
    // Constructor
    public function _Super_${service.name?cap_first}()
    {
        // initialize service control
<#if service.getAnnotationItem("ActionScriptGeneration", "rootURL")??>
        _serviceControl = new HTTPMultiService("${service.getAnnotationItem('ActionScriptGeneration', 'rootURL')}"); 
<#else>
        _serviceControl = new HTTPMultiService(); 
</#if>          
<#if (service.getFunctions().size() > 0)>
         var operations:Array = new Array();
         var operation:Operation;  
         var argsArray:Array;       
         
    <#list service.getFunctions() as function>   
    	<#-- support for remote function mapping -->
    	<#if function.getAnnotationItem("ActionScriptGeneration", "remoteFunction")??>
			<#assign remoteFunctionName>${function.getAnnotationItem("ActionScriptGeneration", "remoteFunction")}</#assign>
		<#else>
			<#assign remoteFunctionName>${function.getName()}</#assign>
		</#if>                
         operation = new Operation(null, "${remoteFunctionName}");
         <#if function.getAnnotationItem("ActionScriptGeneration", "url")??>
         operation.url = "${function.getAnnotationItem("ActionScriptGeneration", "url")}";
         </#if>
         <#if function.getAnnotationItem("ActionScriptGeneration", "method")??>
         operation.method = "${function.getAnnotationItem("ActionScriptGeneration", "method")}";
         </#if>         
         <#if ((function.getSignature().getArity() > 0) && (!(function.getAnnotationItem("ActionScriptGeneration", "requestBodyParameter")?? && function.getAnnotationItem("ActionScriptGeneration", "requestBodyParameter") == "true"))) >
         argsArray = new Array(${util.getServiceArgs(function.getSignature(), true, null)});
         operation.argumentNames = argsArray;         
         </#if>
         <#if  function.getAnnotationItem("ActionScriptGeneration", "SerializationFilter")??>
         operation.serializationFilter = ${serializerMap.get(function.getAnnotationItem("ActionScriptGeneration", "SerializationFilter"))};
         </#if>
         <#-- Add Properties annotation group if present -->
         <#if  function.getAnnotationGroup("Properties")??>
         operation.properties = new Object();
         	<#list function.getAnnotationGroup("Properties").entrySet() as annotEntry>
         operation.properties["${annotEntry.getKey()}"] = "${annotEntry.getValue()}";
         	</#list>
         </#if>
         <#-- Content Type at function level-->
         <#if  function.getAnnotationItem("ActionScriptGeneration", "contentType")??>
         operation.contentType = "${function.getAnnotationItem("ActionScriptGeneration", "contentType")}";
         </#if>             
         <#-- Add resultType or resultElementType -->
         <@getResultType function/>
         operations.push(operation);
    
    </#list>
         _serviceControl.operationList = operations;  
         <#if  service.getAnnotationItem("ActionScriptGeneration", "SerializationFilter")??>
         _serviceControl.serializationFilter = ${serializerMap.get(service.getAnnotationItem("ActionScriptGeneration", "SerializationFilter"))};
         </#if>   
         <#-- Content Type at service level-->
         <#if  service.getAnnotationItem("ActionScriptGeneration", "contentType")??>
         _serviceControl.contentType = "${service.getAnnotationItem("ActionScriptGeneration", "contentType")}";
         </#if>           
</#if>      
${channelSetAndDestination}
${dminits}    
${dmops}                      
         model_internal::initialize();
    }
</#assign>
</#if>
<#if serviceType == "WebService">
<#assign webServiceHeader>
[ExcludeClass]
internal class _Super_${service.name} extends WebServiceWrapper
{
    ${dmvars} 
    // Constructor
    public function _Super_${service.name?cap_first}()
    {
        // initialize service control
         _serviceControl = new WebService();
         <#-- Schema Type Registry annotations -->          
         <#if service.getAnnotationGroup("ASGenerationWSDLTypeMap")??>
            <#list service.getAnnotationGroup("ASGenerationWSDLTypeMap").entrySet() as annot>
         		<#if typeHelper.addImport(annot.getKey(), false) == "_NONE_">
         		</#if>  
         SchemaTypeRegistry.getInstance().registerClass(new QName("${annot.getValue()?substring(0, annot.getValue()?index_of(','))}", "${annot.getValue()?substring(annot.getValue()?index_of(',')+1)}"), ${annot.getKey()}); 
            </#list>
         </#if>   
         <#-- Schema Type Registry colelction annotations -->      
         <#if service.getAnnotationGroup("ASGenerationWSDLCollectionTypes")??>
         	<#if typeHelper.addImport("mx.collections.ArrayCollection", false) == "_NONE_">
         	</#if>  
            <#list service.getAnnotationGroup("ASGenerationWSDLCollectionTypes").entrySet() as annot>
         SchemaTypeRegistry.getInstance().registerCollectionClass(new QName("${annot.getValue()?substring(0, annot.getValue()?index_of(','))}", "${annot.getValue()?substring(annot.getValue()?index_of(',')+1)}"), ArrayCollection); 
            </#list>         
         </#if>
         <#-- Build operations set-->
<#if (service.getFunctions().size() > 0)>
         var operations:Object = new Object();
         var operation:Operation;         
         
    <#list service.getFunctions() as function>
    	<#-- support for remote function mapping -->             
    	<#if function.getAnnotationItem("ActionScriptGeneration", "remoteFunction")??>
			<#assign remoteFunctionName>${function.getAnnotationItem("ActionScriptGeneration", "remoteFunction")}</#assign>
		<#else>
			<#assign remoteFunctionName>${function.getName()}</#assign>
		</#if>          
         operation = new Operation(null, "${remoteFunctionName}");
         <@getResultType function/>
         operations["${remoteFunctionName}"] = operation;
    
    </#list>
         _serviceControl.operations = operations;              
</#if>

${channelSetAndDestination}  

         <#-- Set WSDL-service if it is annotated-->
<#if service.getAnnotationItem("ActionScriptGeneration", "WSDL-service")??>
         _serviceControl.service = "${service.getAnnotationItem("ActionScriptGeneration", "WSDL-service")}";
</#if>  
         <#-- Set WSDL-port if it is annotated-->
<#if service.getAnnotationItem("ActionScriptGeneration", "WSDL-port")??>
         _serviceControl.port = "${service.getAnnotationItem("ActionScriptGeneration", "WSDL-port")}";
</#if>  
         <#-- Set WSDL if it is annotated-->
<#if service.getAnnotationItem("ActionScriptGeneration", "WSDL")??>
         _serviceControl.wsdl = "${service.getAnnotationItem("ActionScriptGeneration", "WSDL")}";
</#if>         
         <#-- loadWSDL if it or the Destination is annotated -->
<#if service.getAnnotationItem("ActionScriptGeneration", "WSDL")?? || service.getAnnotationItem("ActionScriptGeneration", "Destination")??>
         _serviceControl.loadWSDL();
</#if>   
${dminits}      
${dmops}     
         model_internal::initialize();
    }
</#assign>   
</#if>
<#if serviceType == "AbstractServiceExtension">
<#assign abstractServiceExtensionHeader>
[ExcludeClass]
internal class _Super_${service.name} extends AbstractServiceWrapper
{
   /**
     * wrapped ${typeHelper.addImport(service.getAnnotationItem("ActionScriptGeneration", "ServiceControl"), true)}
     * 
     * @private
     */
    protected var _serviceControl:${service.getAnnotationItem("ActionScriptGeneration", "ServiceControl")};
    
   /**
    * Returns the wrapped ${service.getAnnotationItem("ActionScriptGeneration", "ServiceControl")} instance.
    * 
    * @return the wrapped ${service.getAnnotationItem("ActionScriptGeneration", "ServiceControl")} instance.
    *  
    */ 
    override public function get serviceControl():AbstractService
    {
        return _serviceControl;   
    }        
    ${dmvars} 
    // Constructor
    public function _Super_${service.name?cap_first}()
    {
        // initialize service control
         _serviceControl = new ${service.getAnnotationItem("ActionScriptGeneration", "ServiceControl")}();
${channelSetAndDestination}       
${dminits}      
${dmops}     
         model_internal::initialize();
    }
</#assign>  
</#if> 
<#-- Body -->
<#assign classbody>    
<#list service.getFunctions() as function>	
	<#assign isManaged><#if functionMap?? && functionMap.get(function.getName())??>yes<#else>no</#if></#assign> 
	<#if isManaged == "yes">
    	<#assign funcType>${functionMap.get(function.getName()).getFunctionType()}</#assign>
    </#if>
     <#-- assign parameter mapping type -->
	<#if serviceType == "RemoteObject" && service.getAnnotationItem("ActionScriptGeneration", "LCES-Service")??>
		<#assign paramsType>Named</#assign>
	<#else>
		<#if service.getAnnotationItem("ActionScriptGeneration", "ParamsType")??>
			<#assign paramsType>${service.getAnnotationItem("ActionScriptGeneration", "ParamsType")}</#assign>
		<#else>
			<#assign paramsType>None</#assign>
		</#if>
	</#if>    
    <#-- assign return type and case needed for managed create, update, and get -->
    <#if isManaged == "yes" && (funcType == "create" || funcType == "update" || funcType == "get")>
    	<#assign returnType>ItemReference</#assign>
    	<#assign seeClass>mx.data.ItemReference</#assign>
    	<#assign cast> as ItemReference</#assign>
    <#else>
    	<#assign returnType>AsyncToken</#assign>
    	<#assign seeClass>mx.rpc.AsyncToken</#assign>
    	<#assign cast> </#assign>
    </#if>                     
    <#-- remove startIndex and numItems params if paging is enabled for this fill function - supplied automatically by DMS --> 
    <#if isManaged == "yes" && funcType == "fillFunc" && functionMap.get(function.getName()).isPagingEnabled()>                       
		<#assign funcArgs>${util.removePagingParamsFromSignature(function, typeHelper, termFormatter.getBuildContext(), functionMap.get(function.getName()).isPositionalPagingParams())}</#assign>
		<#assign paramList>${util.removePagingParamsFromParamList(util.getServiceArgs(function.getSignature(), false, paramsType), functionMap.get(function.getName()).isPositionalPagingParams())}</#assign>
    <#-- origItem and changes from update function - supplied automatically by DMS --> 
	<#elseif isManaged == "yes" && funcType == "update">
		<#assign funcArgs>${util.removeUpdateParamsFromSignature(typeHelper.format(function.getSignature()), functionMap.get(function.getName()).getEntity())}</#assign>	
		<#assign paramList>${util.removeUpdateParamsFromParamList(util.getServiceArgs(function.getSignature(), false, paramsType))}</#assign>
	<#else>
		<#if function.getAnnotationGroup("OpaqueClarification")?? >
			<#assign funcArgs>${typeHelper.format(function.getSignature(), function.getAnnotationGroup("OpaqueClarification"))}</#assign>
			<#if typeHelper.clearOpaqueClarification()></#if>
		<#else>
			<#assign funcArgs>${typeHelper.format(function.getSignature())}</#assign>
		</#if>		
		<#assign paramList>${util.getServiceArgs(function.getSignature(), false, paramsType)}</#assign>
    </#if>
    <#-- support for remote function mapping -->
    <#if function.getAnnotationItem("ActionScriptGeneration", "remoteFunction")??>
        <#assign remoteFunctionName>${function.getAnnotationItem("ActionScriptGeneration", "remoteFunction")}</#assign>
    <#else>
        <#assign remoteFunctionName>${function.getName()}</#assign>
    </#if>    
    <#-- If we have an LCES service with an "_Async" suffix, do not generate a synchronous invocation function for it as the operation
    cannot be invoked syncrhonously anyway and the generation could cause collisions with other functions -->
    <#if !(serviceType == "RemoteObject" && service.getAnnotationItem("ActionScriptGeneration", "LCES-Service")?? 
           && (function.getName()?length > 6) && (function.getName()?substring((function.getName()?length - 6), function.getName()?length) == "_Async"))>
	/**
	  * This method is a generated wrapper used to call the '${function.getName()}' operation. It returns an ${returnType} whose 
	  * result property will be populated with the result of the operation when the server response is received. 
	  * To use this result from MXML code, define a CallResponder component and assign its token property to this method's return value. 
	  * You can then bind to CallResponder.lastResult or listen for the CallResponder.result or fault events.
      *
      * @see ${seeClass}
      * @see mx.rpc.CallResponder 
      *
      * @return an ${returnType} whose result property will be populated with the result of the operation when the server response is received.
	  */          
	public function ${function.getName()}${funcArgs} : ${returnType}
	{
	<#if serviceType == "WebService">
		model_internal::loadWSDLIfNecessary();
	</#if>
		var _internal_operation:AbstractOperation = _serviceControl.getOperation("${remoteFunctionName}");
		var _internal_token:${returnType} = _internal_operation.send(${paramList})${cast};

		return _internal_token;
	}   
	</#if> <#-- end of LCES, RemoteObject service and function ends in _Async -->
	
	<#-- if this is an LCES-Service, add an asynchronous counterpart for each function -->
	<#if serviceType == "RemoteObject" && service.getAnnotationItem("ActionScriptGeneration", "LCES-Service")??>
    /**
      * This method is a generated wrapper used to asynchronously call the '${function.getName()}' operation. It returns an ${returnType} whose 
      * result property will be populated with the result of the operation, a JobId, when the server response is received. 
      * To use this result from MXML code, define a CallResponder component and assign its token property to this method's return value. 
      * You can then bind to CallResponder.lastResult or listen for the CallResponder.result or fault events.
      *
      * @see ${seeClass}
      * @see mx.rpc.CallResponder 
      *
      * @return an ${returnType} whose result property will be populated with the result of the operation, a JobId, when the server response is received.
      */          
    public function ${function.getName()}_Async${funcArgs} : ${returnType}
    {
        var _internal_operation:AbstractOperation = _serviceControl.getOperation("${remoteFunctionName}_Async");
        var _internal_token:${returnType} = _internal_operation.send(${paramList})${cast};

        return _internal_token;
    }   	
	</#if>

</#list>
}
</#assign>
<#-- ====================================================================================== -->
/**
 * This is a generated class and is not intended for modfication.  To customize behavior
 * of this service wrapper you may modify the generated sub-class of this class - ${service.name}.as.
 */
<#if packageMap.get(service.getName())??>
package ${packageMap.get(service.getName())}
<#else>
package ${model.name}
</#if>
{
import mx.rpc.AsyncToken;
import com.adobe.fiber.core.model_internal;
import mx.rpc.AbstractOperation;
<#list typeHelper.getImports() as imp>
    <#if imp??> 
import ${imp};
    </#if>
</#list>
<#-- Add an import of data manager if any entities are managed by this service -->
<#if managedEntities??>
import mx.data.RPCDataManager;
import mx.data.ManagedOperation;
import mx.data.ManagedAssociation;
import mx.data.ManagedQuery;
import mx.data.ItemReference;
</#if>
<#if serviceType == "RemoteObject">
import mx.collections.ItemResponder;
import mx.rpc.remoting.RemoteObject; 
import mx.rpc.remoting.Operation;
import com.adobe.fiber.services.wrapper.RemoteObjectServiceWrapper;
import com.adobe.fiber.valueobjects.AvailablePropertyIterator;
import com.adobe.serializers.utility.TypeUtility;

${remoteObjectHeader}
</#if>
<#if serviceType == "WebService">
import mx.rpc.soap.mxml.WebService;
import mx.rpc.soap.mxml.Operation;
import com.adobe.fiber.services.wrapper.WebServiceWrapper;
    <#if service.getAnnotationGroup("ASGenerationWSDLTypeMap")?? || service.getAnnotationGroup("ASGenerationWSDLCollectionTypes")??>
import mx.rpc.xml.SchemaTypeRegistry;    
    </#if>

${webServiceHeader}
</#if>
<#if serviceType == "HTTPMultiService">
import com.adobe.fiber.services.wrapper.HTTPServiceWrapper;
import mx.rpc.http.HTTPMultiService;
import mx.rpc.http.Operation;
<#list serializerMap.entrySet() as entry>
import ${entry.getKey()};
</#list>
${httpMultiHeader}
</#if>
<#if serviceType == "AbstractServiceExtension">
import com.adobe.fiber.services.wrapper.AbstractServiceWrapper;
import mx.rpc.AbstractService;
${abstractServiceExtensionHeader}
</#if>
${classbody}
}
